# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-14 08:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0082_konferencja_opcjonalne'),
    ]

    operations = [
        migrations.AddField(
            model_name='patent_autor',
            name='afiliuje',
            field=models.BooleanField(default=True, help_text='Afiliuje \n    się do jednostki podanej w przypisaniu'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle_autor',
            name='afiliuje',
            field=models.BooleanField(default=True, help_text='Afiliuje \n    się do jednostki podanej w przypisaniu'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte_autor',
            name='afiliuje',
            field=models.BooleanField(default=True, help_text='Afiliuje \n    się do jednostki podanej w przypisaniu'),
        ),
        migrations.AlterField(
            model_name='patent_autor',
            name='zatrudniony',
            field=models.BooleanField(default=False, help_text='Pracownik \n    jednostki podanej w przypisaniu'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle_autor',
            name='zatrudniony',
            field=models.BooleanField(default=False, help_text='Pracownik \n    jednostki podanej w przypisaniu'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte_autor',
            name='zatrudniony',
            field=models.BooleanField(default=False, help_text='Pracownik \n    jednostki podanej w przypisaniu'),
        ),
    ]



# SELECT
# 	id,
# 	rok,
# 	tytul_oryginalny,
# 	informacje,
# 	opis_bibliograficzny_cache,
# 	opis_bibliograficzny_zapisani_autorzy_cache
# FROM
# 	bpp_rekord_mat
# WHERE
# 	opis_bibliograficzny_cache LIKE '%W: pod red%';

# -*- encoding: utf-8 -*-
import argparse
import csv
import logging

import Levenshtein
from django.core.management import BaseCommand
from django.db import transaction
from openpyxl import load_workbook

from bpp.models import (
    Zrodlo,
    Rodzaj_Zrodla,
    Wydawnictwo_Zwarte,
    Wydawnictwo_Ciagle,
    TO_REDAKTOR,
)


class Command(BaseCommand):
    help = "Wyrzuca 'pod red.' z pola informacje, gdy jest już redaktor"

    def wyrzuc_pod_red_z_informacji(self, klass):
        """
        :type klass: bpp.models.Wydawnictwo_Ciagle
        """

        for elem in klass.objects.filter(
            informacje__startswith="pod red.",
            opis_bibliograficzny_cache__contains="W: pod red.",
        ):
            parsed_informacje = [
                x.strip()
                for x in elem.informacje.replace("pod red.", "").strip().split(",")
            ]

            zapisani_autorzy = [
                x.zapisany_jako
                for x in elem.autorzy_set.filter(
                    typ_odpowiedzialnosci__typ_ogolny=TO_REDAKTOR
                )
            ]

            odleglosci = [
                Levenshtein.distance(za, pi)
                for za, pi in zip(zapisani_autorzy, parsed_informacje)
            ]

            yield (
                [
                    elem.pk,
                    elem.tytul_oryginalny,
                    elem.rok,
                    elem.informacje,
                    parsed_informacje,
                    zapisani_autorzy,
                    odleglosci,
                ]
            )

    @transaction.atomic
    def handle(self, *args, **options):
        f = open("res.csv", "w")
        writer = csv.writer(f)
        writer.writerow(
            [
                "PK",
                "Tytul",
                "Rok",
                "Informacje",
                "Informacje przeanalizowane",
                "zapisani autorzy",
                "odleglosci",
            ]
        )
        for elem in Wydawnictwo_Zwarte, Wydawnictwo_Ciagle:
            for res in self.wyrzuc_pod_red_z_informacji(elem):
                writer.writerow(res)

        f.close()



import pytest
from django.urls import reverse
from raport_slotow.const import DZIALANIE_SLOT, DZIALANIE_WSZYSTKO
from raport_slotow.forms import AutorRaportSlotowForm
from selenium.webdriver.common.keys import Keys


def test_AutorRaportSlotowForm_clean_lata(autor_jan_kowalski):
    af = AutorRaportSlotowForm(
        dict(
            obiekt=autor_jan_kowalski,
            od_roku=2020,
            do_roku=2018,
            minimalny_pk=0,
            dzialanie="wszystko",
            _export="html",
        )
    )
    af.full_clean()
    assert af.has_error("od_roku", code="od_do_zle")


def test_AutorRaportSlotowForm_clean_dzialanie_wszystko(autor_jan_kowalski):
    af = AutorRaportSlotowForm(
        dict(
            obiekt=autor_jan_kowalski,
            od_roku=2018,
            do_roku=2020,
            minimalny_pk=0,
            dzialanie=DZIALANIE_WSZYSTKO,
            slot=15,
            _export="html",
        )
    )
    af.full_clean()
    assert af.has_error("slot", code="nie_podawaj_gdy_wszystko")


def test_AutorRaportSlotowForm_clean_dzialanie_slot_brak(autor_jan_kowalski):
    af = AutorRaportSlotowForm(
        dict(
            obiekt=autor_jan_kowalski,
            od_roku=2018,
            do_roku=2020,
            minimalny_pk=0,
            dzialanie=DZIALANIE_SLOT,
            _export="html",
        )
    )
    af.full_clean()
    assert af.has_error("slot", code="podawaj_gdy_slot")


def test_AutorRaportSlotowForm_clean_dzialanie_slot_zero(autor_jan_kowalski):
    af = AutorRaportSlotowForm(
        dict(
            obiekt=autor_jan_kowalski,
            od_roku=2018,
            do_roku=2020,
            minimalny_pk=0,
            slot=0,
            dzialanie=DZIALANIE_SLOT,
            _export="html",
        )
    )
    af.full_clean()
    assert af.has_error("slot", code="podawaj_gdy_slot")


@pytest.mark.selenium
def test_AutorRaportSlotowForm_javascript(admin_browser, live_server):
    url = live_server.url + reverse("raport_slotow:index")
    # with wait_for_page_load(admin_browser):
    admin_browser.visit(url)
    elem = admin_browser.find_by_id("id_slot")
    for x in elem.type(["1", "2", Keys.TAB], slowly=True):
        pass
    res = admin_browser.find_by_id("id_dzialanie_0")
    assert not res.selected

    for x in elem.type([Keys.BACKSPACE, Keys.BACKSPACE, Keys.TAB], slowly=True):
        pass
    res = admin_browser.find_by_id("id_dzialanie_0")
    assert res.selected



# Generated by Django 3.0.14 on 2021-10-31 20:03

from django.db import migrations, models

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Log",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("started_on", models.DateTimeField(auto_now_add=True)),
                ("finished_on", models.DateTimeField(blank=True, null=True)),
                ("exitcode", models.SmallIntegerField(blank=True, null=True)),
                ("command_name", models.TextField()),
                (
                    "args",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, null=True
                    ),
                ),
                (
                    "kwargs",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, null=True
                    ),
                ),
                ("stdout", models.TextField(blank=True, null=True)),
                ("stderr", models.TextField(blank=True, null=True)),
            ],
        ),
    ]



import pytest
from django.urls import reverse

from bpp.models import Czas_Udostepnienia_OpenAccess


@pytest.mark.django_db
def test_rest_api_czas_udostepnienia_openaccess_detail(client, openaccess_data):
    pk = Czas_Udostepnienia_OpenAccess.objects.first().pk
    res = client.get(reverse("api_v1:czas_udostepnienia_openaccess-detail", args=(pk,)))
    assert res.status_code == 200


@pytest.mark.django_db
def test_rest_api_czas_udostepnienia_openaccess_list(client, openaccess_data):
    res = client.get(reverse("api_v1:czas_udostepnienia_openaccess-list"))
    assert res.status_code == 200



# Generated by Django 3.0.14 on 2021-07-25 20:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0279_usun_ost_akt_pbn"),
    ]

    operations = [
        migrations.AlterField(
            model_name="charakter_formalny",
            name="rodzaj_pbn",
            field=models.PositiveSmallIntegerField(
                blank=True,
                choices=[
                    (None, "nie eksportuj do PBN"),
                    (1, "artykuł"),
                    (3, "książka"),
                    (2, "rozdział"),
                    (4, "postępowanie"),
                ],
                default=None,
                help_text="Pole określające, czy wydawnictwa posiadające dany charakter formalny zostaną włączone\n        do eksportu PBN jako artykuły, rozdziały czy książki. ",
                null=True,
                verbose_name="Rodzaj dla PBN",
            ),
        ),
    ]



import json

import pytest
from django.urls import reverse
from model_bakery import baker

from fixtures import pbn_journal_json, pbn_pageable_json, pbn_publication_json
from pbn_api.client import PBN_GET_PUBLICATION_BY_ID_URL, PBN_SEARCH_PUBLICATIONS_URL
from pbn_api.const import PBN_GET_JOURNAL_BY_ID
from pbn_api.models import Journal, Publication

from bpp.const import PBN_UID_LEN
from bpp.models import Autor_Dyscyplina, Wydawnictwo_Zwarte
from bpp.models.autor import Autor
from bpp.models.konferencja import Konferencja
from bpp.views.autocomplete import (
    AdminNavigationAutocomplete,
    AutorAutocomplete,
    GlobalNavigationAutocomplete,
    JednostkaMixin,
    PublicAutorAutocomplete,
)
from bpp.views.autocomplete.pbn_api import JournalAutocomplete, PublicationAutocomplete

VALUES = [
    "Zi%C4%99ba+%5C",
    "Zi%C4%99ba+%5C \\",
    'fa\\"fa',
    "'",
    "fa ' fa",
    " ' fa",
    " fa '",
    "fa\\'fa",
    "Zięba \\",
    "Test ; test",
    "test (test)",
    "test & test",
    "test &",
    "& test",
    "; test",
    "test ;",
    ":*",
    ":",
    ":* :* *: *:",
    "",
    "\\",
    "123 \\ 123",
    "\\ 123",
    "123 \\",
    "|K",
]
AUTOCOMPLETES = [
    "bpp:public-autor-autocomplete",
    "bpp:jednostka-widoczna-autocomplete",
    "bpp:dyscyplina-autocomplete",
]


@pytest.mark.django_db
@pytest.mark.parametrize("autocomplete_name", AUTOCOMPLETES)
@pytest.mark.parametrize("qstr", VALUES)
def test_autocomplete_bug_1(autocomplete_name, qstr, client):
    res = client.get(reverse(autocomplete_name), data={"q": qstr})
    assert res.status_code == 200


@pytest.mark.django_db
def test_admin_konferencje():
    "Upewnij się, że konferencje wyskakują w AdminAutoComplete"
    k = baker.make(Konferencja, nazwa="test 54")
    a = AdminNavigationAutocomplete()
    a.q = "test 54"
    assert k in a.get_queryset()


@pytest.mark.django_db
def test_public_autor_autocomplete_bug_1():
    a = PublicAutorAutocomplete()
    a.q = "a (b)"
    assert list(a.get_queryset()) is not None

    a.q = "a\tb"
    assert list(a.get_queryset()) is not None


def test_dyscyplina_naukowa_przypisanie_autocomplete(
    app, autor_jan_kowalski, dyscyplina1, dyscyplina2, rok
):
    res = app.get(reverse("bpp:dyscyplina-naukowa-przypisanie-autocomplete"))
    assert res.json["results"][0]["text"] == "Podaj autora"

    f = json.dumps({"autor": autor_jan_kowalski.id})
    res = app.get(
        reverse("bpp:dyscyplina-naukowa-przypisanie-autocomplete"), {"forward": f}
    )
    assert res.json["results"][0]["text"] == "Podaj rok"

    f = json.dumps({"autor": autor_jan_kowalski.id, "rok": "fa"})
    res = app.get(
        reverse("bpp:dyscyplina-naukowa-przypisanie-autocomplete"), {"forward": f}
    )
    assert res.json["results"][0]["text"] == "Nieprawidłowy rok"

    f = json.dumps({"autor": autor_jan_kowalski.id, "rok": -10})
    res = app.get(
        reverse("bpp:dyscyplina-naukowa-przypisanie-autocomplete"), {"forward": f}
    )
    assert res.json["results"][0]["text"] == "Nieprawidłowy rok"

    f = json.dumps({"autor": autor_jan_kowalski.id, "rok": rok})
    res = app.get(
        reverse("bpp:dyscyplina-naukowa-przypisanie-autocomplete"), {"forward": f}
    )
    assert res.json["results"][0]["text"] == "Brak przypisania dla roku %i" % rok

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski,
        rok=rok,
        dyscyplina_naukowa=dyscyplina2,
        subdyscyplina_naukowa=dyscyplina1,
    )

    f = json.dumps({"autor": autor_jan_kowalski.id, "rok": rok})
    res = app.get(
        reverse("bpp:dyscyplina-naukowa-przypisanie-autocomplete"), {"forward": f}
    )
    assert res.json["results"][0]["text"] == "druga dyscyplina"

    f = json.dumps({"autor": autor_jan_kowalski.id, "rok": rok})
    res = app.get(
        reverse("bpp:dyscyplina-naukowa-przypisanie-autocomplete"),
        {"forward": f, "q": "memetyka"},
    )
    assert res.json["results"][0]["text"] == "memetyka stosowana"


def test_dyscyplina_naukowa_przypisanie_autocomplete_brak_drugiej(
    app, autor_jan_kowalski, dyscyplina1, dyscyplina2, rok
):

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski,
        rok=rok,
        dyscyplina_naukowa=dyscyplina2,
    )

    f = json.dumps({"autor": autor_jan_kowalski.id, "rok": rok})
    res = app.get(
        reverse("bpp:dyscyplina-naukowa-przypisanie-autocomplete"), {"forward": f}
    )
    assert res.json["results"][0]["text"] == "druga dyscyplina"


def test_wydawca_autocomplete(admin_client):
    res = admin_client.get(reverse("bpp:wydawca-autocomplete"))
    assert res.status_code == 200


def test_wydawnictwo_nadrzedne_autocomplete(admin_client):
    admin_client.get(reverse("bpp:wydawnictwo-nadrzedne-autocomplete") + "?q=test")


def test_publicwydawnictwo_nadrzedne_autocomplete(admin_client, ksiazka):
    ksiazka.tytul_oryginalny = "test 123"
    ksiazka.save()

    res = admin_client.get(
        reverse("bpp:public-wydawnictwo-nadrzedne-autocomplete") + "?q=test"
    )
    assert not json.loads(res.content)["results"]

    baker.make(Wydawnictwo_Zwarte, wydawnictwo_nadrzedne=ksiazka)
    res = admin_client.get(
        reverse("bpp:public-wydawnictwo-nadrzedne-autocomplete") + "?q=test"
    )
    assert len(json.loads(res.content)["results"]) == 1


def test_tsquery_bug1(admin_client):
    # Tak, ktoś kiedyś wkleił taki oto ciąg znaków w wyszukiwanie.
    res = admin_client.get(
        "/bpp/public-autor-autocomplete/?q=Nazwa%20raportu%3A%09raport%20slot%C3%B3w"
        "%20-%20autor%09%09%09%09%09%09%20Autor%3A%09Krawiec%20Marcela%09%09%09%09%09"
        "%09%20Dyscyplina%3A%09rolnictwo%20i%20ogrodnictwo%20(4.2)%09%09%09%09%09%09"
        "%20Od%20roku%3A%092019%09%09%09%09%09%09%20Do%20roku%3A%092020%09%09%09%09%09"
        "%09%20Wygenerowano%3A%092020-02-14%207%3A19%3A44%09%09%09%09%09%09%20Wersja%20"
        "oprogramowania%20BPP%09202002.15%09%09%09%09%09%09%20%09%09%09%09%09%09%09%20"
        "Tytu%C5%82%20oryginalny%09Autorzy%09Rok%09%C5%B9r%C3%B3d%C5%82o%09Dyscyplina%09"
        "Punkty%20PK%09Punkty%20dla%20autora%09Slot%20Impact%20of%20seed%20light%20"
        "stimulation%20on%20the%20mechanical%20strength%20and%20photosynthetic%20pigments"
        "%20content%20in%20the%20scorzonera%20leaves%09Anna%20Ciupak%2C%20Agata%20"
        "Dziwulska-Hunek%2C%20Marcela%20Krawiec%2C%20Bo%C5%BCena%20G%C5%82adyszewska"
        "%092019%09Acta%20Agrophysica%09rolnictwo%20i%20ogrodnictwo%20(4.2)%0920%095"
        "%090%2C25%20EFFECT%20OF%20NATURAL%20FERTILIZATION%20AND%20CALCIUM%20CARBONATE"
        "%20ON%20YIELDING%20AND%20BIOLOGICAL%20VALUE%20OF%20THYME%20(%3Ci%3EThymus%20"
        "vulgaris%3C%2Fi%3E%20L.)%09Katarzyna%20Dzida%2C%20Zenia%20Micha%C5%82oj%C4%87"
        "%2C%20Zbigniew%20Jarosz%2C%20Karolina%20Pitura%2C%20Natalia%20Skubij%2C%20Daniel"
        "%20Skubij%2C%20Marcela%20Krawiec%092019%09Acta%20Scientiarum%20Polonorum-Hortorum"
        "%20Cultus%09rolnictwo%20i%20ogrodnictwo%20(4.2)%0970%0911%2C8322%090%2C169%20"
        "CHEMICAL%20%20AND%20%20NONCHEMICAL%20%20CONTROL%20%20OF%20%20WEEDS%20%20IN%20"
        "%20THE%20%20CULTIVATION%20%20OF%20%20LEMON%20%20BALM%20%20FOR%20%20SEEDS%09Marcela"
        "%20Krawiec%2C%20Andrzej%20Borowy%2C%20Katarzyna%20Dzida%092019%09Acta%20Scientiarum"
        "%20Polonorum-Hortorum%20Cultus%09rolnictwo%20i%20ogrodnictwo%20(4.2)%0970%0923"
        "%2C3333%090%2C3333"
    )
    assert res.status_code == 200


@pytest.mark.django_db
def test_AutorAutocomplete_create_bug_1():
    assert Autor.objects.count() == 0

    def autocomplete(s):
        a = AutorAutocomplete()
        a.q = s
        res = a.create_object(s)
        return res

    res = autocomplete("  fubar")

    assert res.pk == -1
    assert Autor.objects.count() == 0

    res = autocomplete("  fubar baz quux")
    assert res.pk != -1
    assert Autor.objects.count() == 1
    assert Autor.objects.first().nazwisko == "Fubar"
    assert Autor.objects.first().imiona == "Baz Quux"


def test_JednostkaMixin_get_result_label(jednostka):
    jednostka.wydzial = None
    assert JednostkaMixin().get_result_label(jednostka)


@pytest.mark.django_db
@pytest.mark.parametrize(
    "klass", [AdminNavigationAutocomplete, GlobalNavigationAutocomplete]
)
def test_NavigationAutocomplete_no_queries(
    django_assert_max_num_queries,
    klass,
    jednostka,
    zrodlo,
    wydawnictwo_ciagle,
    wydawnictwo_zwarte,
    autor_jan_kowalski,
    autor_jan_nowak,
    rf,
    admin_user,
):
    req = rf.get("/", data={"q": "Je"})
    req.user = admin_user
    with django_assert_max_num_queries(11):
        a = klass()
        a.request = req
        a.q = "Je"  # literka
        a.get(req)

    with django_assert_max_num_queries(13):
        a = klass()
        a.q = "T" * 24  # PBN UID
        a.request = req
        a.get(req)

    with django_assert_max_num_queries(13):
        a = klass()
        a.request = req
        a.q = "T" * 19  # orcid
        a.get(req)


@pytest.mark.django_db
def test_PublicationAutocomplete_get_create_option(rf, admin_user):
    ac = PublicationAutocomplete()
    ac.request = rf.get("/")
    ac.request.user = admin_user
    ac.q = "1" * PBN_UID_LEN
    res = ac.get_create_option({"object_list": []}, "1" * PBN_UID_LEN)
    assert str(res[0]["text"]).find("Pobierz rekord o UID") >= 0


@pytest.mark.django_db
def test_PublicationAutocomplete_get_queryset():
    ac = PublicationAutocomplete()

    baker.make(
        Publication,
        pk="1" * PBN_UID_LEN,
        **pbn_publication_json(2020, title="Takie tam"),
    )

    ac.q = "1" * PBN_UID_LEN
    assert ac.get_queryset().exists()
    ac.q = "Takie tam"
    assert ac.get_queryset().exists()


@pytest.mark.django_db
def test_JournalAutocomplete_get_queryset():
    ac = JournalAutocomplete()

    baker.make(
        Journal,
        pk="1" * PBN_UID_LEN,
        **pbn_journal_json(title="Test"),
    )

    ac.q = "1" * PBN_UID_LEN
    assert ac.get_queryset().exists()
    ac.q = "Test"
    assert ac.get_queryset().exists()


@pytest.mark.django_db
def test_PublicationAutocomplete_create_object(
    pbn_uczelnia, pbn_client, rf, admin_user, pbn_serwer
):
    ac = PublicationAutocomplete()
    ac.request = rf.get("/")
    ac.request.user = admin_user

    ROK = 2020
    UID_REKORDU = "1" * PBN_UID_LEN
    ISBN = "123-123-123-123"

    pub1 = pbn_publication_json(ROK, mongoId=UID_REKORDU, isbn=ISBN)
    pbn_serwer.expect_request(PBN_SEARCH_PUBLICATIONS_URL).respond_with_json(
        pbn_pageable_json([pub1])
    )
    pbn_serwer.expect_request(
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=UID_REKORDU)
    ).respond_with_json(pub1)

    assert ac.create_object(UID_REKORDU)


@pytest.mark.django_db
def test_PublicationAutocomplete_post(
    pbn_uczelnia, pbn_client, rf, admin_user, pbn_serwer
):
    ac = PublicationAutocomplete()

    ROK = 2020
    UID_REKORDU = "1" * PBN_UID_LEN
    ISBN = "123-123-123-123"

    pub1 = pbn_publication_json(ROK, mongoId=UID_REKORDU, isbn=ISBN)
    pbn_serwer.expect_request(PBN_SEARCH_PUBLICATIONS_URL).respond_with_json(
        pbn_pageable_json([pub1])
    )
    pbn_serwer.expect_request(
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=UID_REKORDU)
    ).respond_with_json(pub1)

    ac.request = rf.post("/", data={"text": UID_REKORDU})
    ac.request.user = admin_user
    assert ac.create_object(UID_REKORDU)


@pytest.mark.django_db
def test_JournalAutocomplete_post(pbn_uczelnia, pbn_client, rf, admin_user, pbn_serwer):
    ac = JournalAutocomplete()

    UID_REKORDU = "1" * PBN_UID_LEN
    ISSN = "4567-4567"

    pub1 = pbn_journal_json(mongoId=UID_REKORDU, issn=ISSN)
    # pbn_serwer.expect_request(PBN_SEARCH_PUBLICATIONS_URL).respond_with_json(
    #     pbn_pageable_json([pub1])
    # )
    pbn_serwer.expect_request(
        PBN_GET_JOURNAL_BY_ID.format(id=UID_REKORDU)
    ).respond_with_json(pub1)

    ac.request = rf.post("/", data={"text": UID_REKORDU})
    ac.request.user = admin_user
    assert ac.create_object(UID_REKORDU)



from io import BytesIO

import PyPDF2
import pytest
from django.urls import reverse
from openpyxl import load_workbook

from raport_slotow import const
from raport_slotow.forms import AutorRaportSlotowForm
from raport_slotow.views import SESSION_KEY


def test_raport_slotow_formularz(admin_client):
    res = admin_client.get(reverse("raport_slotow:index"))
    assert res.status_code == 200


def test_raport_slotow_autor_brak_danych(admin_client, autor_jan_kowalski, rok):
    url = reverse(
        "raport_slotow:raport",
    )

    dane_raportu = {
        "obiekt": autor_jan_kowalski.pk,
        "od_roku": rok,
        "do_roku": rok,
        "dzialanie": const.DZIALANIE_WSZYSTKO,
        "minimalny_pk": 0,
        "slot": None,
        "_export": "html",
    }

    form = AutorRaportSlotowForm(dane_raportu)
    assert form.is_valid(), form._errors

    s = admin_client.session
    s.update({SESSION_KEY: dane_raportu})
    s.save()

    res = admin_client.get(url)
    assert res.status_code == 200
    assert "Brak danych" in res.rendered_content

    res = admin_client.get(url + "?_export=xlsx")
    assert res.status_code == 200
    wb = load_workbook(BytesIO(res.content))
    assert len(wb.get_sheet_names()) > 0


def test_raport_slotow_autor_sa_dane_eksport_wszystkiego(
    admin_client,
    autor_jan_kowalski,
    rekord_slotu,
    rok,
):
    url = reverse("raport_slotow:raport")

    dane_raportu = {
        "obiekt": autor_jan_kowalski.pk,
        "od_roku": rok,
        "do_roku": rok,
        "dzialanie": const.DZIALANIE_WSZYSTKO,
        "minimalny_pk": 0,
        "slot": None,
        "_export": "html",
    }
    s = admin_client.session
    s.update({SESSION_KEY: dane_raportu})
    s.save()

    res = admin_client.get(url)
    assert res.status_code == 200
    assert "Brak danych" not in res.rendered_content

    res = admin_client.get(url + "?_export=xlsx")
    assert res.status_code == 200
    wb = load_workbook(BytesIO(res.content))
    assert len(wb.get_sheet_names()) > 0


def test_raport_slotow_autor_sa_dane_eksport_wszystkiego_do_pdf(
    admin_client,
    autor_jan_kowalski,
    rekord_slotu,
    rok,
):
    url = reverse("raport_slotow:raport")

    dane_raportu = {
        "obiekt": autor_jan_kowalski.pk,
        "od_roku": rok,
        "do_roku": rok,
        "dzialanie": const.DZIALANIE_WSZYSTKO,
        "minimalny_pk": 0,
        "slot": None,
        "_export": "html",
    }
    s = admin_client.session
    s.update({SESSION_KEY: dane_raportu})
    s.save()

    res = admin_client.get(url + "?_export=pdf")
    assert res.status_code == 200
    pdfReader = PyPDF2.PdfFileReader(BytesIO(res.content))
    assert pdfReader.numPages >= 1


def test_raport_slotow_autor_zbieraj_slot(
    admin_client, autor_jan_kowalski, rekord_slotu, rok
):
    url = reverse("raport_slotow:raport")

    dane_raportu = {
        "obiekt": autor_jan_kowalski.pk,
        "od_roku": rok,
        "do_roku": rok,
        "dzialanie": const.DZIALANIE_SLOT,
        "minimalny_pk": 0,
        "slot": 20,
        "_export": "html",
    }
    s = admin_client.session
    s.update({SESSION_KEY: dane_raportu})
    s.save()

    res = admin_client.get(url)
    assert res.status_code == 200
    assert "Brak danych" not in res.rendered_content

    res = admin_client.get(url + "?_export=xlsx")
    assert res.status_code == 200
    wb = load_workbook(BytesIO(res.content))
    assert len(wb.get_sheet_names()) > 0


def test_raport_slotow_autor_wartosci_poczatkowe(admin_client):
    url = reverse("raport_slotow:index")
    res = admin_client.get(url, dict(od_roku=5000))
    assert b"5000" in res.content


@pytest.mark.parametrize(
    "dzialanie,slot", [(const.DZIALANIE_WSZYSTKO, None), (const.DZIALANIE_SLOT, 20)]
)
def test_raport_slotow_autor_sa_dane_minimalny_pk(
    admin_client, autor_jan_kowalski, rekord_slotu, rok, dzialanie, slot
):
    w = rekord_slotu.rekord
    w.punkty_pk = 10
    w.save()

    url = reverse("raport_slotow:raport")

    dane_raportu = {
        "obiekt": autor_jan_kowalski.pk,
        "od_roku": rok,
        "do_roku": rok,
        "dzialanie": dzialanie,
        "minimalny_pk": 0,
        "slot": slot,
        "_export": "html",
    }
    s = admin_client.session
    s.update({SESSION_KEY: dane_raportu})
    s.save()

    res = admin_client.get(url)
    assert res.status_code == 200
    assert "Brak danych" not in res.rendered_content

    dane_raportu = {
        "obiekt": autor_jan_kowalski.pk,
        "od_roku": rok,
        "do_roku": rok,
        "dzialanie": dzialanie,
        "minimalny_pk": 200,
        "slot": slot,
        "_export": "html",
    }
    s = admin_client.session
    s.update({SESSION_KEY: dane_raportu})
    s.save()

    res = admin_client.get(url)
    assert res.status_code == 200
    assert "Brak danych" in res.rendered_content



import pytest

from pbn_api.client import PBNClient, RequestsTransport
from pbn_api.tests.utils import MockTransport


def test_RequestsTransport_get(mocker):
    m = mocker.MagicMock()
    m.status_code = 200
    rg = mocker.patch("requests.get", return_value=m)

    t = RequestsTransport("foo", "bar", "onet.pl")
    t.get("foobar", {"foo": "bar"})
    rg.assert_called_once()


def test_PBNClient_get_pages_warning():
    t = MockTransport(
        {
            "/api/v1/conferences/page?size=10": {},
        }
    )
    c = PBNClient(t)
    with pytest.warns(RuntimeWarning, match="did not return a paged"):
        c.get_conferences()


def test_PBNClient_get_conferences():
    t = MockTransport(
        {
            "/api/v1/conferences/page?size=10": {
                "content": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                "number": 0,
                "numberOfElements": 10,
                "pageable": {
                    "offset": 0,
                    "pageNumber": 0,
                    "pageSize": 10,
                    "paged": True,
                    "sort": {"sorted": False, "unsorted": True},
                    "unpaged": False,
                },
                "size": 10,
                "totalElements": 10,
                "totalPages": 1,
            },
            "/api/v1/conferences/page?size=10&page=1": {
                "content": [],
                "number": 0,
                "numberOfElements": 10,
                "pageable": {
                    "offset": 0,
                    "pageNumber": 0,
                    "pageSize": 10,
                    "paged": True,
                    "sort": {"sorted": False, "unsorted": True},
                    "unpaged": False,
                },
                "size": 10,
                "totalElements": 10,
                "totalPages": 1,
            },
        }
    )
    c = PBNClient(t)
    assert list(c.get_conferences()) == [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]



from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import Fieldset, Hidden, Layout, Submit
from django.forms import ModelForm, inlineformset_factory

from import_dyscyplin.models import Import_Dyscyplin, Kolumna


class Import_Dyscyplin_KolumnaForm(ModelForm):
    class Meta:
        model = Import_Dyscyplin
        fields = [
            "web_page_uid",
        ]

    def __init__(self, *args, **kw):
        super(Import_Dyscyplin_KolumnaForm, self).__init__(*args, **kw)
        helper = FormHelper(self)
        helper.form_class = "custom"
        helper.layout = Layout(
            Hidden("web_page_uid", ""),
        )
        self.helper = helper


class KolumnaForm(ModelForm):
    class Meta:
        model = Kolumna
        fields = ["nazwa_w_pliku", "rodzaj_pola"]


class KolumnaFormSetHelper(FormHelper):
    template = "bootstrap/table_inline_formset.html"

    def __init__(self, *args, **kwargs):
        super(KolumnaFormSetHelper, self).__init__(*args, **kwargs)
        self.form_method = "post"
        self.add_input(
            Submit("submit", "Zatwierdź i przejdź dalej", css_class="button success")
        )
        self.render_required_fields = True


KolumnaFormSet = inlineformset_factory(
    Import_Dyscyplin, Kolumna, form=KolumnaForm, can_delete=False, extra=0
)


class Import_DyscyplinForm(ModelForm):
    class Meta:
        fields = ("plik", "rok")
        model = Import_Dyscyplin

    def __init__(self, *args, **kw):
        super(Import_DyscyplinForm, self).__init__(*args, **kw)
        helper = FormHelper(self)
        helper.form_class = "custom"
        helper.layout = Layout(
            Fieldset(
                "Zaimportuj plik",
                "plik",
                "rok",
                Hidden("web_page_uid", ""),
                Submit("submit", "Wyślij", css_id="id_submit"),
            )
        )
        self.helper = helper



import pytest

from import_common.exceptions import HeaderNotFoundException
from import_common.util import rename_duplicate_columns, znajdz_naglowek


def test_znajdz_naglowek_dobry(test1_xlsx):
    row, no = znajdz_naglowek(test1_xlsx)
    assert no == 1


def test_znajdz_naglowek_zly(test2_bad_header_xlsx):
    with pytest.raises(HeaderNotFoundException):
        znajdz_naglowek(test2_bad_header_xlsx)


def test_znajdz_naglowek_default(default_xlsx):
    znajdz_naglowek(default_xlsx)


def test_rename_duplicate_columns():
    i = ["issn", "test", "issn", "jeszcze", "raz", "test", "issn"]
    o = ["issn", "test", "issn_2", "jeszcze", "raz", "test_2", "issn_3"]

    assert rename_duplicate_columns(i) == o



# Generated by Django 3.0.11 on 2021-04-06 02:56

from django.db import migrations, models

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0006_sciencist"),
    ]

    operations = [
        migrations.CreateModel(
            name="Publication",
            fields=[
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_updated_on", models.DateTimeField(auto_now=True)),
                (
                    "mongoId",
                    models.CharField(max_length=32, primary_key=True, serialize=False),
                ),
                ("status", models.CharField(db_index=True, max_length=32)),
                ("verificationLevel", models.CharField(max_length=32)),
                ("verified", models.BooleanField(default=False)),
                ("versions", django.contrib.postgres.fields.jsonb.JSONField()),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



# Generated by Django 3.0.14 on 2021-10-31 20:22

from django.db import migrations, models

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ("tee", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="log",
            name="exitcode",
        ),
        migrations.AddField(
            model_name="log",
            name="exit_code",
            field=models.SmallIntegerField(
                blank=True,
                help_text="If value returned by\n    django.core.management.call_command is None, this will be zero. If value returned by d.c.m.call_command\n    is an int, this will be that int. If different, this is set to -1 and exit_value field is set. ",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="log",
            name="exit_value",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True,
                help_text="JSON-encoded value, returned by\n    django.core.management.call_command function, if different than None or an int. ",
                null=True,
            ),
        ),
    ]



import os
from argparse import FileType

from django.core.management import BaseCommand

from ewaluacja2021.reports import load_data, rekordy
from ewaluacja2021.util import autor2fn
from ewaluacja2021.xlsy import AutorskiXLSX, CalosciowyXLSX, WypelnienieXLSX

from bpp.models import Autor
from bpp.util import pbar


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("wejscie", type=FileType("r"))
        parser.add_argument("--katalog-wyjsciowy", type=str, default=None)

    def handle(self, wejscie, katalog_wyjsciowy, liczba_n=None, *args, **options):

        dane = load_data(wejscie)

        if katalog_wyjsciowy is None:
            katalog_wyjsciowy = wejscie.name.replace(".json", "_output")

        if not os.path.exists(katalog_wyjsciowy):
            os.mkdir(katalog_wyjsciowy)

        rekordy_danych = rekordy(dane)

        WypelnienieXLSX(
            "AAA_wypelnienie_slotow",
            rekordy=rekordy_danych,
            dane=dane,
            katalog_wyjsciowy=katalog_wyjsciowy,
        ).zrob()

        CalosciowyXLSX(
            "AAA_rekordy",
            rekordy=rekordy_danych,
            dane=dane,
            katalog_wyjsciowy=katalog_wyjsciowy,
        ).zrob()

        for autor in pbar(
            Autor.objects.filter(pk__in=(x.autor_id for x in rekordy(dane))),
            label="Dane autorow...",
        ):
            rekordy_autora = rekordy_danych.filter(autor_id=autor.id)
            AutorskiXLSX(
                autor=autor,
                title=autor2fn(autor),
                rekordy=rekordy_autora,
                dane=dane,
                katalog_wyjsciowy=katalog_wyjsciowy,
            ).zrob()



# Generated by Django 3.0.14 on 2021-11-22 00:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ewaluacja2021", "0007_auto_20211110_0002"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="zamowienienaraport",
            options={"ordering": ("-ostatnio_zmodyfikowany",)},
        ),
        migrations.AlterField(
            model_name="zamowienienaraport",
            name="rodzaj",
            field=models.CharField(
                choices=[
                    ("plecakowy", "plecakowy"),
                    ("plecakowy_bez_limitu", "plecakowy bez limitu na uczelnię"),
                    ("genetyczny", "genetyczny"),
                    ("genetyczny_z_odpinaniem", "genetyczny z odpinaniem"),
                ],
                max_length=25,
                verbose_name="Rodzaj algorytmu",
            ),
        ),
    ]



from datetime import date

from django.core.serializers.json import DjangoJSONEncoder
from django.db import IntegrityError, models, transaction
from django.db.models import CASCADE, Count, JSONField, Q
from django.urls import reverse
from django_fsm import GET_STATE, FSMField, transition
from model_utils.models import TimeStampedModel

from import_common.exceptions import (
    BadNoOfSheetsException,
    HeaderNotFoundException,
    ImproperFileException,
)
from import_common.util import znajdz_naglowek

from bpp.fields import YearField
from bpp.models import Autor, Autor_Dyscyplina, Dyscyplina_Naukowa, Jednostka, Wydzial

from django_bpp.settings.base import AUTH_USER_MODEL


def obecny_rok():
    return date.today().year


class Kolumna(models.Model):
    parent = models.ForeignKey("import_dyscyplin.Import_Dyscyplin", models.CASCADE)

    class RODZAJ:
        POMIJAJ = "pomiń"

        TYTUL = "tytuł"

        NAZWISKO = "nazwisko"
        IMIE = "imię"
        ORCID = "orcid"
        PBN_ID = "pbn_id"
        NAZWA_JEDNOSTKI = "nazwa jednostki"
        WYDZIAL = "wydział"

        DYSCYPLINA = "dyscyplina"
        KOD_DYSCYPLINY = "kod dyscypliny"
        PROCENT_DYSCYPLINY = "procent dyscypliny"

        SUBDYSCYPLINA = "subdyscyplina"
        KOD_SUBDYSCYPLINY = "kod subdyscypliny"
        PROCENT_SUBDYSCYPLINY = "procent subdyscypliny"

    RODZAJE = [
        RODZAJ.TYTUL,
        RODZAJ.NAZWISKO,
        RODZAJ.IMIE,
        RODZAJ.ORCID,
        RODZAJ.PBN_ID,
        RODZAJ.NAZWA_JEDNOSTKI,
        RODZAJ.WYDZIAL,
        RODZAJ.DYSCYPLINA,
        RODZAJ.KOD_DYSCYPLINY,
        RODZAJ.PROCENT_DYSCYPLINY,
        RODZAJ.SUBDYSCYPLINA,
        RODZAJ.KOD_SUBDYSCYPLINY,
        RODZAJ.PROCENT_SUBDYSCYPLINY,
        RODZAJ.POMIJAJ,
    ]

    kolejnosc = models.PositiveSmallIntegerField()
    nazwa_w_pliku = models.CharField(max_length=250)
    rodzaj_pola = models.CharField(max_length=50, choices=zip(RODZAJE, RODZAJE))

    class Meta:
        ordering = [
            "kolejnosc",
        ]


def guess_rodzaj(s):
    s = s.lower().replace(" ", "")
    if s in [
        "tytuł_stopień",
        "tytuł",
        "tytułnaukowy",
        "tytuł/stopień",
        "tytułstopień",
        "stopieńnaukowy",
        "stopień",
        "stopien",
        "tytul",
        "tytulnaukowy",
    ]:
        return Kolumna.RODZAJ.TYTUL

    if s in ["nazwisko", "nazwiska"]:
        return Kolumna.RODZAJ.NAZWISKO
    if s in ["imię", "imie", "imiona"]:
        return Kolumna.RODZAJ.IMIE
    if s in ["orcid", "identyfikatororcid", "ident.orcid"]:
        return Kolumna.RODZAJ.ORCID
    if s in ["jednostka", "nazwajednostki", "nazwa_jednostki"]:
        return Kolumna.RODZAJ.NAZWA_JEDNOSTKI
    if s in ["wydzial", "wydz.", "wydział"]:
        return Kolumna.RODZAJ.WYDZIAL

    if s in ["dyscyplina", "dyscyplina1", "dyscyplinagłówna", "dyscyplinaglowna"]:
        return Kolumna.RODZAJ.DYSCYPLINA
    if s in [
        "kod_dyscypliny",
        "koddyscypliny",
        "koddyscyplinyglownej",
        "koddyscyplinygłównej",
        "kod1",
        "koddyscypliny1",
    ]:
        return Kolumna.RODZAJ.KOD_DYSCYPLINY
    if s in [
        "procent_dyscypliny",
        "procentdyscypliny",
        "procentdyscypliny1",
        "procentdyscyplinygłównej",
        "procentdyscyplinyglownej",
        "udziałprocentowy1",
    ]:
        return Kolumna.RODZAJ.PROCENT_DYSCYPLINY

    if s in [
        "subdyscyplina",
        "dyscyplina2",
        "dyscyplinapoboczna",
        "dyscyplinapoboczna",
    ]:
        return Kolumna.RODZAJ.SUBDYSCYPLINA
    if s in [
        "kod_subdyscypliny",
        "kodsubdyscypliny",
        "koddyscyplinypoboczej",
        "koddyscyplinydrugiej",
        "kod2",
        "koddyscypliny2",
    ]:
        return Kolumna.RODZAJ.KOD_SUBDYSCYPLINY
    if s in [
        "procent_subdyscypliny",
        "procentsubdyscypliny",
        "procentdyscypliny2",
        "procentdyscyplinypobocznej",
        "udziałprocentowy2",
    ]:
        return Kolumna.RODZAJ.PROCENT_SUBDYSCYPLINY

    if s in [
        "pbn_id",
        "pbn-id",
        "pbnid",
        "pbn_id",
        "pbn*id",
        "identyfikatorpbn",
        "ident.pbn",
    ]:
        return Kolumna.RODZAJ.PBN_ID

    return Kolumna.RODZAJ.POMIJAJ


class Import_Dyscyplin(TimeStampedModel):
    class STAN:
        NOWY = "nowy"
        OKRESLANIE_OPCJI_IMPORTU = "określanie opcji importu"
        OPCJE_IMPORTU_OKRESLONE = "opcje importu określone"
        BLEDNY = "błędny"
        PRZEANALIZOWANY = "przeanalizowany"

        ZINTEGROWANY = "zintegrowany"

    STANY = (
        STAN.NOWY,
        STAN.OKRESLANIE_OPCJI_IMPORTU,
        STAN.OPCJE_IMPORTU_OKRESLONE,
        STAN.PRZEANALIZOWANY,
        STAN.BLEDNY,
        STAN.ZINTEGROWANY,
    )

    owner = models.ForeignKey(AUTH_USER_MODEL, CASCADE)
    web_page_uid = models.CharField(max_length=36, blank=True, null=True)
    web_page_uid.__doc__ = (
        "UUID4 strony internetowej która może być używana do notyfikacji. "
    )

    task_id = models.CharField(max_length=36, blank=True, null=True)
    task_id.__doc__ = "celery.uuid() z identyfikatorem zadania"

    plik = models.FileField()
    rok = YearField(default=obecny_rok)
    stan = FSMField(default=STAN.NOWY, choices=zip(STANY, STANY), protected=True)

    bledny = models.BooleanField(default=False)
    info = models.TextField(blank=True, null=True)

    wiersz_naglowka = models.PositiveSmallIntegerField(null=True, blank=True)

    kolumny = models.ManyToManyField(Kolumna)

    @transition(
        field=stan,
        source=STAN.NOWY,
        target=GET_STATE(
            lambda self: self.STAN.BLEDNY
            if self.bledny
            else self.STAN.OKRESLANIE_OPCJI_IMPORTU,
            states=[STAN.BLEDNY, STAN.OKRESLANIE_OPCJI_IMPORTU],
        ),
        on_error=STAN.BLEDNY,
    )
    def stworz_kolumny(self):

        try:
            kolumny, wiersz = znajdz_naglowek(self.plik.path)
        except ImproperFileException as e:
            self.bledny = True
            self.info = "niepoprawny plik - %s" % e
            return
        except BadNoOfSheetsException:
            self.bledny = True
            self.info = "Plik musi zawierać tylko jeden arkusz"
            return
        except HeaderNotFoundException:
            self.bledny = True
            self.info = "Nagłówek nie został znaleziony"
            return

        self.kolumna_set.all().delete()
        for n, kolumna in enumerate(kolumny):
            Kolumna.objects.create(
                parent=self,
                kolejnosc=n,
                nazwa_w_pliku=kolumna,
                rodzaj_pola=guess_rodzaj(kolumna) or Kolumna.RODZAJ.POMIJAJ,
            )
        self.wiersz_naglowka = wiersz
        self.save()

    @transition(
        field=stan,
        source=STAN.OKRESLANIE_OPCJI_IMPORTU,
        target=STAN.OPCJE_IMPORTU_OKRESLONE,
    )
    def zatwierdz_kolumny(self):
        self.save()

    @transition(
        field=stan,
        source=STAN.OPCJE_IMPORTU_OKRESLONE,
        target=GET_STATE(
            lambda self: "błędny" if self.bledny else "przeanalizowany",
            states=[STAN.BLEDNY, STAN.PRZEANALIZOWANY],
        ),
        on_error=STAN.BLEDNY,
    )
    def przeanalizuj(self):
        from import_dyscyplin.core import przeanalizuj_plik_xls

        przeanalizuj_plik_xls(self.plik.path, parent=self)

    def wiersze(self):
        return Import_Dyscyplin_Row.objects.filter(parent=self)

    def poprawne_wiersze_do_integracji(self):
        return (
            self.wiersze()
            .exclude(autor_id=None)
            .exclude(stan=Import_Dyscyplin_Row.STAN.BLEDNY)
            .exclude(stan=Import_Dyscyplin_Row.STAN.ZINTEGROWANY)
        )

    def niepoprawne_wiersze(self):
        return self.wiersze().filter(
            Q(autor_id=None) | Q(stan=Import_Dyscyplin_Row.STAN.BLEDNY)
        )

    def distinct_info_dla_qs(self, qs):
        return (
            qs.order_by("info").values("info").annotate(icount=Count("info")).distinct()
        )

    def niepoprawne_wiersze_przyczyny(self):
        return self.distinct_info_dla_qs(self.niepoprawne_wiersze())

    def zintegrowane_wiersze_przyczyny(self):
        return self.distinct_info_dla_qs(self.zintegrowane_wiersze())

    def zintegrowane_wiersze(self):
        return self.wiersze().filter(stan=Import_Dyscyplin_Row.STAN.ZINTEGROWANY)

    def integruj_dyscypliny(self):
        # order_by jest w poniższym zapytaniu potrzebny, żeby pozbyć się
        # problemu z domyślnym porządkiem sortowania, wywołanym przez
        # Import_Dyscyplin_Row.Meta - wówczas distinct powinien zawierać
        # również kolumny po których sortujemy, a to jest nam zbędne, więc:
        for elem in (
            self.wiersze()
            .order_by("dyscyplina")
            .values(
                "dyscyplina", "kod_dyscypliny", "subdyscyplina", "kod_subdyscypliny"
            )
            .distinct()
        ):

            if not elem["dyscyplina"] or not elem["kod_dyscypliny"]:
                continue

            try:
                d, _c = Dyscyplina_Naukowa.objects.get_or_create(
                    nazwa=elem["dyscyplina"], kod=elem["kod_dyscypliny"]
                )
            except IntegrityError:
                for r in self.wiersze().filter(
                    dyscyplina=elem["dyscyplina"], kod_dyscypliny=elem["kod_dyscypliny"]
                ):
                    r.stan = Import_Dyscyplin_Row.STAN.BLEDNY
                    r.info = (
                        "Nie można zintegrować nazwy i kodu dyscypliny. W bazie danych istnieje "
                        "już rekord o takiej nazwie i innym kodzie lub o takim kodzie ale o innej "
                        "nazwie."
                    )
                    r.save()

                continue

            for r in self.wiersze().filter(
                dyscyplina=elem["dyscyplina"], kod_dyscypliny=elem["kod_dyscypliny"]
            ):
                r.dyscyplina_naukowa = d
                r.save()

            if not elem["subdyscyplina"] or not elem["kod_subdyscypliny"]:
                continue

            try:
                sd, _c = Dyscyplina_Naukowa.objects.get_or_create(
                    nazwa=elem["subdyscyplina"], kod=elem["kod_subdyscypliny"]
                )
            except IntegrityError:
                for r in self.wiersze().filter(
                    subdyscyplina=elem["subdyscyplina"],
                    kod_subdyscypliny=elem["kod_subdyscypliny"],
                ):
                    r.stan = Import_Dyscyplin_Row.STAN.BLEDNY
                    r.info = (
                        "Nie można zintegrować nazwy i kodu subdyscypliny. W bazie danych istnieje "
                        "już rekord o takiej nazwie i innym kodzie lub o takim kodzie ale o innej "
                        "nazwie."
                    )
                    r.save()
                continue

            for r in self.wiersze().filter(
                subdyscyplina=elem["subdyscyplina"],
                kod_subdyscypliny=elem["kod_subdyscypliny"],
            ):
                r.subdyscyplina_naukowa = sd
                r.save()

    def sprawdz_czy_konieczne(self):
        """Sprawdza wszystkie wiersze, po przeanalizowaniu oraz po
        integracji dyscyplin, czy wprowadzanie ich jest konieczne do bazy. To znaczy,
        jeżeli istnieją już takie przypisania do autora i dyscypliny, to oznacz
        takie rekordy adekwatnie.
        """
        for elem in self.poprawne_wiersze_do_integracji().select_related():
            try:
                Autor_Dyscyplina.objects.get(
                    autor=elem.autor,
                    rok=self.rok,
                    dyscyplina_naukowa=elem.subdyscyplina_naukowa,
                    procent_dyscypliny=elem.procent_dyscypliny,
                    subdyscyplina_naukowa=elem.subdyscyplina_naukowa,
                    procent_subdyscypliny=elem.procent_subdyscypliny,
                )
                elem.stan = Import_Dyscyplin_Row.STAN.BLEDNY
                elem.info = "Istnieje identyczne przypisanie"
                elem.save()
                continue
            except Autor_Dyscyplina.DoesNotExist:

                try:
                    Autor_Dyscyplina.objects.get(autor=elem.autor, rok=self.rok)
                except Autor_Dyscyplina.DoesNotExist:
                    if (
                        elem.dyscyplina_naukowa is None
                        and elem.subdyscyplina_naukowa is None
                    ):
                        elem.stan = Import_Dyscyplin_Row.STAN.BLEDNY
                        elem.info = "To przypisanie zostało już usunięte"
                        elem.save()

            if elem.dyscyplina_naukowa is not None:
                elem.info = "Skasuję przypisanie na ten rok"
                elem.save()

    def id_zdublowanych_autorow(self):
        return (
            self.poprawne_wiersze_do_integracji()
            .values_list("autor_id", flat=True)
            .order_by("autor_id")
            .annotate(acount=Count("autor_id"))
            .filter(acount__gt=1)
            .distinct()
        )

    def sprawdz_czy_poprawne(self):
        """Sprawdza, czy są wiersze z podwójnym (tym samym) autorem w imporcie"""
        for elem in self.id_zdublowanych_autorow():
            for r in self.poprawne_wiersze_do_integracji().filter(autor_id=elem):
                r.stan = Import_Dyscyplin_Row.STAN.BLEDNY
                r.info = "Więcej, niż jeden wiersz z pliku XLS został dopasowany do tego autora w systemie."
                r.save()

    @transition(
        field=stan,
        source=STAN.PRZEANALIZOWANY,
        target=GET_STATE(
            lambda self: "błędny" if self.bledny else "zintegrowany",
            states=[STAN.BLEDNY, STAN.ZINTEGROWANY],
        ),
        on_error=STAN.BLEDNY,
    )
    def integruj_wiersze(self):
        return self._integruj_wiersze()

    def _integruj_wiersze(self):
        for elem in self.poprawne_wiersze_do_integracji().select_related():

            if elem.dyscyplina_naukowa is None:
                # Kasowanie
                # elem["subdyscyplina_naukowa"] został już wcześniej sprawdzony.
                Autor_Dyscyplina.objects.get(autor=elem.autor, rok=self.rok).delete()
                elem.info = f"Usunięto przypisanie do dyscyplin za rok {self.rok}"
                elem.stan = Import_Dyscyplin_Row.STAN.ZINTEGROWANY
                elem.save()
                continue

            try:
                ad = Autor_Dyscyplina.objects.get(autor=elem.autor, rok=self.rok)

                changed = False
                for attr in [
                    "dyscyplina_naukowa",
                    "procent_dyscypliny",
                    "subdyscyplina_naukowa",
                    "procent_subdyscypliny",
                ]:
                    source = getattr(elem, attr)
                    target = getattr(ad, attr)

                    if source != target:
                        setattr(ad, attr, source)
                        if elem.info is None:
                            elem.info = ""

                        elem.info += (
                            "Istniejące %s dla roku %i zmieniono z %s na %s. "
                            % (
                                attr.replace("_", " "),
                                self.rok,
                                target or "[brak]",
                                source or "[brak]",
                            )
                        )

                        changed = True

                if changed:
                    ad.save()
                else:
                    elem.info = "stan w bazie zgodny jak w pliku importu"

            except Autor_Dyscyplina.DoesNotExist:
                Autor_Dyscyplina.objects.create(
                    autor=elem.autor,
                    rok=self.rok,
                    dyscyplina_naukowa=elem.dyscyplina_naukowa,
                    procent_dyscypliny=elem.procent_dyscypliny,
                    subdyscyplina_naukowa=elem.subdyscyplina_naukowa,
                    procent_subdyscypliny=elem.procent_subdyscypliny,
                )
                elem.info = "nowe przypisanie dla %i: %s, %s." % (
                    self.rok,
                    elem.dyscyplina_naukowa,
                    elem.subdyscyplina_naukowa,
                )

            elem.stan = Import_Dyscyplin_Row.STAN.ZINTEGROWANY
            elem.save()

        Autor_Dyscyplina.objects.ukryj_nieuzywane()

    def delete(self, *args, **kw):
        transaction.on_commit(lambda instance=self: instance.plik.delete(False))
        return super().delete(*args, **kw)

    class Meta:
        ordering = ("-modified",)


class Import_Dyscyplin_Row(models.Model):
    class STAN:
        NOWY = "nowy"
        BLEDNY = "błędny"
        ZINTEGROWANY = "zintegrowany"

    STANY = (STAN.NOWY, STAN.BLEDNY, STAN.ZINTEGROWANY)

    parent = models.ForeignKey(Import_Dyscyplin, CASCADE)

    stan = models.CharField(
        max_length=50, choices=zip(STANY, STANY), default=STAN.NOWY, db_index=True
    )
    info = models.TextField(blank=True, null=True)

    row_no = models.IntegerField()
    original = JSONField(encoder=DjangoJSONEncoder)

    nazwisko = models.CharField(max_length=200)
    imiona = models.CharField(max_length=200)

    autor = models.ForeignKey(Autor, null=True, on_delete=models.SET_NULL)

    nazwa_jednostki = models.CharField(max_length=512, blank=True, null=True)
    jednostka = models.ForeignKey(Jednostka, null=True, on_delete=models.SET_NULL)

    nazwa_wydzialu = models.CharField(max_length=512, blank=True, null=True)
    wydzial = models.ForeignKey(Wydzial, null=True, on_delete=models.SET_NULL)

    dyscyplina = models.CharField(max_length=200, db_index=True, blank=True, null=True)
    kod_dyscypliny = models.CharField(
        max_length=20, db_index=True, blank=True, null=True
    )
    procent_dyscypliny = models.DecimalField(
        max_digits=5, decimal_places=2, null=True, blank=True
    )
    dyscyplina_naukowa = models.ForeignKey(
        Dyscyplina_Naukowa,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
    )

    subdyscyplina = models.CharField(
        max_length=200, null=True, blank=True, db_index=True
    )
    kod_subdyscypliny = models.CharField(
        max_length=20, null=True, blank=True, db_index=True
    )
    procent_subdyscypliny = models.DecimalField(
        max_digits=5, decimal_places=2, null=True, blank=True
    )
    subdyscyplina_naukowa = models.ForeignKey(
        Dyscyplina_Naukowa,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="+",
    )

    def serialize_dict(self):
        ret = {
            "nazwisko": self.nazwisko,
            "imiona": self.imiona,
            "jednostka": self.original.get("nazwa jednostki", ""),
            "wydzial": self.original.get("wydzia", ""),
            "info": self.info,
            "dyscyplina": f"{self.dyscyplina} ({self.kod_dyscypliny})",
            "procent_dyscypliny": self.procent_dyscypliny or "",
            "subdyscyplina": f"{self.subdyscyplina} ({self.kod_subdyscypliny})",
            "procent_subdyscypliny": self.procent_subdyscypliny or "",
            "dopasowanie_autora": "-",
        }

        for elem in "dyscyplina", "subdyscyplina":
            if ret[elem] == " ()":
                ret[elem] = ""

        if self.autor is not None:
            ret["autor_slug"] = self.autor.slug
            ret["dopasowanie_autora"] = "<a target=_blank href='{}'>{} {}</a>".format(
                reverse("bpp:browse_autor", args=(self.autor.slug,)),
                self.autor.nazwisko,
                self.autor.imiona,
            )

        return ret

    class Meta:
        ordering = ("nazwisko", "imiona")



import pytest

from import_common.normalization import normalize_kod_dyscypliny


@pytest.mark.parametrize(
    "i,o",
    [
        ("101_0", "1.1"),
        ("111_0", "1.11"),
        ("407", "4.7"),
        ("411", "4.11"),
        ("4.1", "4.1"),
    ],
)
def test_normalize_kod_dyscypliny(i, o):
    assert normalize_kod_dyscypliny(i) == o



"""W tym pakiecie znajdują się procedury generujące raporty, które są dostępne
"od ręki" -- generowane za pomocą WWW"""
from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import ButtonHolder
from crispy_forms_foundation.layout import Column as F4Column
from crispy_forms_foundation.layout import Fieldset, Hidden, Layout, Row, Submit
from dal import autocomplete
from django import forms
from django.core import validators
from django_tables2.export.export import TableExport

from bpp.models import Uczelnia, Wydzial
from bpp.models.autor import Autor
from bpp.models.struktura import Jednostka


def ustaw_rok(rok, lata):
    lata = list(lata)
    try:
        rok.field.min_value = lata[0]
    except IndexError:
        pass

    try:
        rok.field.max_value = lata[-1]
        rok.field.initial = lata[-2]
    except IndexError:
        pass

    if rok.field.max_value is not None:
        rok.field.validators.append(validators.MaxValueValidator(rok.field.max_value))
    if rok.field.min_value is not None:
        rok.field.validators.append(validators.MinValueValidator(rok.field.min_value))


class RaportJednostekForm(forms.Form):
    jednostka = forms.ModelChoiceField(
        queryset=Jednostka.objects.filter(widoczna=True),
        widget=autocomplete.ModelSelect2(url="bpp:jednostka-widoczna-autocomplete"),
    )

    od_roku = forms.IntegerField()
    do_roku = forms.IntegerField()
    output = forms.BooleanField(
        label="Pobierz jako plik Microsoft Word", required=False
    )

    def __init__(self, lata, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_method = "post"
        self.helper.form_action = "#RaportJednostek"

        self.helper.layout = Layout(
            Fieldset(
                "Raport jednostek",
                Row(F4Column("jednostka", css_class="large-12 small-12")),
                Row(
                    F4Column("od_roku", css_class="large-6 small-6"),
                    F4Column("do_roku", css_class="large-6 small-6"),
                ),
                Row(F4Column("output", css_class="large-12 small-12")),
                Hidden("report", "raport-jednostek"),
            ),
            ButtonHolder(
                Submit("submit", "Wyświetl", css_class="button"),
            ),
        )

        super().__init__(*args, **kwargs)
        ustaw_rok(self["od_roku"], lata)
        ustaw_rok(self["do_roku"], lata)


class RaportAutorowForm(forms.Form):
    autor = forms.ModelChoiceField(
        queryset=Autor.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:autor-z-uczelni-autocomplete"),
    )
    od_roku = forms.IntegerField()
    do_roku = forms.IntegerField()
    output = forms.BooleanField(
        label="Pobierz jako plik Microsoft Word", required=False
    )

    def __init__(self, lata, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_method = "post"
        self.helper.form_action = "#RaportAutorow"

        # self.helper.form_action = "./prepare/"
        self.helper.layout = Layout(
            Fieldset(
                "Raport autorów",
                Row(F4Column("autor", css_class="large-12 small-12")),
                Row(
                    F4Column("od_roku", css_class="large-6 small-6"),
                    F4Column("do_roku", css_class="large-6 small-6"),
                ),
                Row(F4Column("output", css_class="large-12 small-12")),
                Hidden("report", "raport-autorow"),
            ),
            ButtonHolder(Submit("submit", "Szukaj", css_class="button white")),
        )

        super().__init__(*args, **kwargs)
        ustaw_rok(self["od_roku"], lata)
        ustaw_rok(self["do_roku"], lata)


class RaportDlaKomisjiCentralnejForm(forms.Form):
    autor = forms.ModelChoiceField(
        queryset=Autor.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:public-autor-autocomplete"),
    )

    rok_habilitacji = forms.IntegerField(required=False)

    def __init__(self, lata, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_method = "post"
        self.helper.form_action = "#RaportDlaKomisjiCentralnej"
        # self.helper.form_action = "./prepare/"
        self.helper.layout = Layout(
            Fieldset(
                "Raport dla Komisji Centralnej",
                "autor",
                "rok_habilitacji",
                Hidden("report", "raport-dla-komisji-centralnej"),
            ),
            ButtonHolder(Submit("submit", "Zamów", css_class="button white")),
        )

        super().__init__(*args, **kwargs)
        ustaw_rok(self["rok_habilitacji"], lata)


class WydzialChoiceField(forms.ModelMultipleChoiceField):
    def label_from_instance(self, obj):
        return obj.nazwa


OUTPUT_FORMATS = [
    ("html", "wyświetl w przeglądarce"),
]

OUTPUT_FORMATS.extend(
    zip(
        list(TableExport.FORMATS.keys()),
        [x.upper() for x in list(TableExport.FORMATS.keys())],
    )
)


class RankingAutorowForm(forms.Form):
    wydzialy = WydzialChoiceField(
        label="Ogranicz do wydziału (wydziałów):",
        required=False,
        widget=forms.SelectMultiple(attrs={"size": "8"}),
        queryset=Wydzial.objects.filter(
            widoczny=True, zezwalaj_na_ranking_autorow=True
        ),
        help_text="Jeżeli nie wybierzesz żadnego wydziału, system wygeneruje "
        "dane dla wszystkich wydziałów. Przytrzymaj przycisk CTRL ("
        "CMD na Maku) gdy klikasz, aby wybrać więcej, niż jeden "
        "wydział lub odznaczyć już zaznaczony wydział. ",
    )

    rozbij_na_jednostki = forms.BooleanField(
        label="Rozbij punktację na jednostki i wydziały",
        required=False,
        initial=lambda: Uczelnia.objects.first().ranking_autorow_rozbij_domyslnie,
    )

    tylko_afiliowane = forms.BooleanField(
        label="Tylko prace afiliowane na jednostki uczelni",
        required=False,
        initial=False,
        help_text="Pokaż tylko prace, gdzie autor afiliował na jednostkę "
        "wchodzącą w struktury uczelni",
    )

    od_roku = forms.IntegerField(initial=Uczelnia.objects.do_roku_default)
    do_roku = forms.IntegerField(initial=Uczelnia.objects.do_roku_default)

    _export = forms.ChoiceField(
        label="Format wyjściowy", required=True, choices=OUTPUT_FORMATS
    )

    def __init__(self, lata, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_method = "post"

        self.helper.layout = Layout(
            Fieldset(
                "Ranking autorów",
                Row(
                    F4Column("od_roku", css_class="large-6 small-6"),
                    F4Column("do_roku", css_class="large-6 small-6"),
                ),
                Row(F4Column("_export", css_class="large-12 small-12")),
                Row(F4Column("rozbij_na_jednostki", css_class="large-12 small-12")),
                Row(F4Column("tylko_afiliowane", css_class="large-12 small-12")),
                Row(F4Column("wydzialy", css_class="large-12 small-12")),
                Hidden("report", "ranking-autorow"),
            ),
            ButtonHolder(Submit("submit", "Otwórz", css_class="button white")),
        )

        super().__init__(*args, **kwargs)
        # ustaw_rok(self['od_roku'], lata)
        # ustaw_rok(self['do_roku'], lata)



from rest_framework import viewsets

from api_v1.serializers.wydawca import Poziom_WydawcySerializer, WydawcaSerializer
from bpp.models import Wydawca
from bpp.models.wydawca import Poziom_Wydawcy


class Poziom_WydawcyViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Poziom_Wydawcy.objects.all()
    serializer_class = Poziom_WydawcySerializer


class WydawcaViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Wydawca.objects.all()
    serializer_class = WydawcaSerializer



# -*- coding: utf-8 -*-


from django.db import models, migrations
from django.db.models import CASCADE, SET_NULL

import integrator2.models.lista_ministerialna
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bpp', '0035_nomorefixtures_data_migration'),
    ]

    operations = [
        migrations.CreateModel(
            name='ListaMinisterialnaElement',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('zanalizowano', models.BooleanField(default=False)),
                ('moze_byc_zintegrowany_automatycznie', models.NullBooleanField(default=None)),
                ('extra_info', models.TextField()),
                ('zintegrowano', models.BooleanField(default=False)),
                ('nazwa', models.TextField()),
                ('issn', models.CharField(max_length=32, null=True, verbose_name='ISSN', blank=True)),
                ('e_issn', models.CharField(max_length=32, null=True, verbose_name='e-ISSN', blank=True)),
                ('punkty_kbn', models.IntegerField()),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ListaMinisterialnaIntegration',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, verbose_name=b'Nazwa pliku')),
                ('file', models.FileField(upload_to=b'integrator2', verbose_name=b'Plik')),
                ('uploaded_on', models.DateTimeField(auto_now_add=True)),
                ('last_updated_on', models.DateTimeField(auto_now=True, auto_now_add=True)),
                ('status', models.IntegerField(default=0, choices=[(0, b'dodany'), (1, b'w trakcie analizy'), (2, b'przetworzony'), (3, b'przetworzony z b\xc5\x82\xc4\x99dami')])),
                ('extra_info', models.TextField()),
                ('year', models.IntegerField(default=integrator2.models.lista_ministerialna.last_year)),
                ('owner', models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=CASCADE)),
            ],
            options={
                'verbose_name': 'Integracja list ministerialnych',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='listaministerialnaelement',
            name='parent',
            field=models.ForeignKey(to='integrator2.ListaMinisterialnaIntegration', on_delete=CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='listaministerialnaelement',
            name='zrodlo',
            field=models.ForeignKey(blank=True, to='bpp.Zrodlo', null=True, on_delete=SET_NULL),
            preserve_default=True,
        ),
    ]



# -*- encoding: utf-8 -*-

from django.conf.urls import url
from django.views.generic.base import TemplateView

from nowe_raporty.views import (
    GenerujRaportDlaJednostki,
    GenerujRaportDlaUczelni,
    GenerujRaportDlaWydzialu,
    UczelniaRaportFormView,
)
from .views import (
    AutorRaportFormView,
    GenerujRaportDlaAutora,
    JednostkaRaportFormView,
    WydzialRaportFormView,
)

urlpatterns = [
    url(
        r"^index/$",
        TemplateView.as_view(template_name="nowe_raporty/index.html"),
        name="index",
    ),
    url(r"^uczelnia/$", UczelniaRaportFormView.as_view(), name="uczelnia_form"),
    url(
        r"uczelnia/(?P<od_roku>\d+)/(?P<do_roku>\d+)/",
        GenerujRaportDlaUczelni.as_view(),
        name="uczelnia_generuj",
    ),
    url(r"^wydzial/$", WydzialRaportFormView.as_view(), name="wydzial_form"),
    url(
        r"wydzial/(?P<pk>\d+)/(?P<od_roku>\d+)/(?P<do_roku>\d+)/",
        GenerujRaportDlaWydzialu.as_view(),
        name="wydzial_generuj",
    ),
    url(r"^jednostka/$", JednostkaRaportFormView.as_view(), name="jednostka_form"),
    url(
        r"jednostka/(?P<pk>\d+)/(?P<od_roku>\d+)/(?P<do_roku>\d+)/",
        GenerujRaportDlaJednostki.as_view(),
        name="jednostka_generuj",
    ),
    url(r"^autor/$", AutorRaportFormView.as_view(), name="autor_form"),
    url(
        r"autor/(?P<pk>\d+)/(?P<od_roku>\d+)/(?P<do_roku>\d+)/",
        GenerujRaportDlaAutora.as_view(),
        name="autor_generuj",
    ),
]



# -*- encoding: utf-8 -*-
from bpp.models.abstract import NazwaISkrot


class Tryb_OpenAccess_Wydawnictwo_Ciagle(NazwaISkrot):
    class Meta:
        verbose_name = "tryb OpenAccess wyd. ciągłych"
        verbose_name_plural = "tryby OpenAccess wyd. ciągłych"
        ordering = ['nazwa']
        app_label = 'bpp'


class Tryb_OpenAccess_Wydawnictwo_Zwarte(NazwaISkrot):
    class Meta:
        verbose_name = "tryb OpenAccess wyd. zwartych"
        verbose_name_plural = "tryby OpenAccess wyd. zwartych"
        ordering = ['nazwa']
        app_label = 'bpp'


class Czas_Udostepnienia_OpenAccess(NazwaISkrot):
    class Meta:
        verbose_name = 'czas udostępnienia OpenAccess'
        verbose_name_plural = 'czasy udostępnienia OpenAccess'
        ordering = ['nazwa']
        app_label = 'bpp'


class Licencja_OpenAccess(NazwaISkrot):
    class Meta:
        verbose_name = 'licencja OpenAccess'
        verbose_name_plural = 'licencja OpenAccess'
        ordering = ['nazwa']
        app_label = 'bpp'

    def webname(self):
        """
        Zwróc nazwę licencji, którą możemy podlinkować na WWW. Generalnie oznacza to
        zmniejszenie znaków i wyrzucenie początkowego CC- ze skrótu. Jeżeli skrót
        nie jest zamienialny na nazwę linku licencji Creative Commons, zwracaj None.
        """
        if self.skrot == "OTHER":
            return

        if self.skrot is not None:
            return self.skrot.lower().replace("cc-", "")


class Wersja_Tekstu_OpenAccess(NazwaISkrot):
    class Meta:
        verbose_name = 'wersja tekstu OpenAccess'
        verbose_name_plural = 'wersje tekstu OpenAccess'
        ordering = ['nazwa']
        app_label = 'bpp'



from django import forms
from django.core.exceptions import ValidationError
from djangoql.admin import DjangoQLSearchMixin

from django.contrib import admin
from django.contrib.postgres.search import TrigramSimilarity

from bpp.admin.filters import PBN_UID_IDObecnyFilter
from bpp.const import PBN_UID_LEN
from bpp.models import Wydawca
from bpp.models.wydawca import Poziom_Wydawcy


class Poziom_WydawcyInlineForm(forms.ModelForm):
    class Meta:
        fields = "__all__"

    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)
        if kw.get("instance"):
            self.fields["rok"].disabled = True

    def clean(self):
        if (
            "wydawca" in self.cleaned_data
            and self.cleaned_data["wydawca"].alias_dla_id is not None
        ):
            raise ValidationError("Nie można przypisywac poziomu wydawcy dla aliasów")
        return self.cleaned_data


class Poziom_WydawcyInline(admin.TabularInline):
    model = Poziom_Wydawcy
    form = Poziom_WydawcyInlineForm
    extra = 1


class MaAliasListFilter(admin.SimpleListFilter):
    title = "ma alias"
    parameter_name = "ma_alias"

    def lookups(self, request, model_admin):
        return [
            ("0", "nie"),
            ("1", "tak"),
        ]

    def queryset(self, request, queryset):
        if self.value() == "0":
            return queryset.filter(alias_dla=None)
        elif self.value() == "1":
            return queryset.exclude(alias_dla=None)


@admin.register(Wydawca)
class WydawcaAdmin(DjangoQLSearchMixin, admin.ModelAdmin):
    djangoql_completion_enabled_by_default = False
    djangoql_completion = True

    search_fields = [
        "nazwa",
        "alias_dla__nazwa",
        "pbn_uid__mongoId",
        "alias_dla__pbn_uid__mongoId",
    ]
    autocomplete_fields = ["alias_dla", "pbn_uid"]
    list_display = [
        "nazwa",
        "alias_dla",
        "ile_aliasow",
        "poziomy_wydawcy",
        "pbn_uid_id",
    ]
    list_filter = [MaAliasListFilter, PBN_UID_IDObecnyFilter]
    inlines = [
        Poziom_WydawcyInline,
    ]

    MIN_TRIGRAM_MATCH = 0.05

    def ile_aliasow(self, obj):
        if obj.ile_aliasow:
            return obj.ile_aliasow
        return

    def poziomy_wydawcy(self, obj):
        if obj.lista_poziomow:
            return "określone"

    def get_search_results(self, request, queryset, search_term):
        if self.search_mode_toggle_enabled() and self.djangoql_search_enabled(request):
            return DjangoQLSearchMixin.get_search_results(
                self, request, queryset, search_term
            )

        if not search_term or (
            len(search_term) == PBN_UID_LEN and search_term.find(" ") == -1
        ):
            return super().get_search_results(request, queryset, search_term)

        queryset = (
            queryset.annotate(similarity=TrigramSimilarity("nazwa", search_term))
            .filter(similarity__gte=self.MIN_TRIGRAM_MATCH)
            .order_by("-similarity")
        )
        return queryset, False



# -*- encoding: utf-8 -*-

from django.apps import apps

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

import pytest

from bpp.tests import (
    add_extra_autor_inline,
    fill_admin_form,
    fill_admin_inline,
    set_element,
    submit_admin_form,
    submitted_form_bad,
    submitted_form_good,
)

from django_bpp.selenium_util import wait_for_page_load

ID = "id_tytul_oryginalny"

pytestmark = [pytest.mark.slow, pytest.mark.selenium]


@pytest.mark.parametrize(
    "klass", ["wydawnictwo_ciagle", "wydawnictwo_zwarte", "patent"]
)
def test_procent_odpowiedzialnosci_baseModel_AutorFormset_jeden_autor(
    asgi_live_server,
    admin_browser,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    typ_odpowiedzialnosci_autor,
    zrodlo,
    charaktery_formalne,
    typy_kbn,
    statusy_korekt,
    jezyki,
    klass,
):
    url = asgi_live_server.url + reverse("admin:bpp_%s_add" % klass)

    with wait_for_page_load(admin_browser):
        admin_browser.visit(url)
    fill_admin_form(admin_browser)
    add_extra_autor_inline(admin_browser)
    fill_admin_inline(
        admin_browser,
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="Kopara",
        procent="100.00",
    )
    submit_admin_form(admin_browser)
    assert submitted_form_good(admin_browser)


@pytest.mark.parametrize(
    "klass", ["wydawnictwo_ciagle", "wydawnictwo_zwarte", "patent"]
)
def test_procent_odpowiedzialnosci_baseModel_AutorFormset_problem_jeden_autor(
    asgi_live_server,
    admin_browser,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    typ_odpowiedzialnosci_autor,
    zrodlo,
    charaktery_formalne,
    typy_kbn,
    statusy_korekt,
    jezyki,
    klass,
):
    url = asgi_live_server.url + reverse("admin:bpp_%s_add" % klass)

    with wait_for_page_load(admin_browser):
        admin_browser.visit(url)
    fill_admin_form(admin_browser)
    add_extra_autor_inline(admin_browser)
    fill_admin_inline(
        admin_browser,
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="Kopara",
        procent="100.01",
    )
    submit_admin_form(admin_browser)
    assert submitted_form_bad(admin_browser)


@pytest.mark.parametrize(
    "klass", ["wydawnictwo_ciagle", "wydawnictwo_zwarte", "patent"]
)
def test_procent_odpowiedzialnosci_baseModel_AutorFormset_dwoch_autorow(
    asgi_live_server,
    admin_browser,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    typ_odpowiedzialnosci_autor,
    zrodlo,
    charaktery_formalne,
    typy_kbn,
    statusy_korekt,
    jezyki,
    klass,
):
    url = asgi_live_server.url + reverse("admin:bpp_%s_add" % klass)

    with wait_for_page_load(admin_browser):
        admin_browser.visit(url)
    fill_admin_form(admin_browser)
    add_extra_autor_inline(admin_browser)
    fill_admin_inline(
        admin_browser,
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="Kopara",
        procent="50.00",
    )
    add_extra_autor_inline(admin_browser, 1)
    fill_admin_inline(
        admin_browser,
        autor=autor_jan_nowak,
        jednostka=jednostka,
        zapisany_jako="Kopara",
        procent="50.00",
        no=1,
    )
    submit_admin_form(admin_browser)
    assert submitted_form_good(admin_browser)


@pytest.mark.parametrize(
    "klass", ["wydawnictwo_ciagle", "wydawnictwo_zwarte", "patent"]
)
def test_procent_odpowiedzialnosci_baseModel_AutorFormset_problem_dwoch_autorow(
    asgi_live_server,
    admin_browser,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    typ_odpowiedzialnosci_autor,
    zrodlo,
    charaktery_formalne,
    typy_kbn,
    statusy_korekt,
    jezyki,
    klass,
):
    url = asgi_live_server.url + reverse("admin:bpp_%s_add" % klass)

    with wait_for_page_load(admin_browser):
        admin_browser.visit(url)
    fill_admin_form(admin_browser)
    add_extra_autor_inline(admin_browser)
    fill_admin_inline(
        admin_browser,
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="Kopara",
        procent="50.00",
    )
    add_extra_autor_inline(admin_browser, 1)
    fill_admin_inline(
        admin_browser,
        autor=autor_jan_nowak,
        jednostka=jednostka,
        zapisany_jako="Kopara",
        procent="50.01",
        no=1,
    )
    submit_admin_form(admin_browser)
    assert submitted_form_bad(admin_browser)


@pytest.mark.parametrize(
    "klass", ["wydawnictwo_ciagle", "wydawnictwo_zwarte", "patent"]
)
def test_procent_odpowiedzialnosci_baseModel_AutorFormset_dobrze_potem_zle_dwoch_autorow(
    asgi_live_server,
    admin_browser,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    typ_odpowiedzialnosci_autor,
    zrodlo,
    charaktery_formalne,
    typy_kbn,
    statusy_korekt,
    jezyki,
    klass,
):
    url = asgi_live_server.url + reverse("admin:bpp_%s_add" % klass)

    with wait_for_page_load(admin_browser):
        admin_browser.visit(url)

    fill_admin_form(admin_browser)

    add_extra_autor_inline(admin_browser)
    fill_admin_inline(
        admin_browser,
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        zapisany_jako="Kopara",
        procent="50.00",
    )

    add_extra_autor_inline(admin_browser)
    fill_admin_inline(
        admin_browser,
        autor=autor_jan_nowak,
        jednostka=jednostka,
        zapisany_jako="Kopara",
        procent="50.00",
        no=1,
    )

    submit_admin_form(admin_browser)

    assert submitted_form_good(admin_browser)

    model = apps.get_app_config("bpp").get_model(klass)

    url = asgi_live_server.url + reverse(
        "admin:bpp_%s_change" % klass, args=(model.objects.first().pk,)
    )

    with wait_for_page_load(admin_browser):
        admin_browser.visit(url)

    set_element(admin_browser, "id_autorzy_set-0-procent", "50.01")

    submit_admin_form(admin_browser)
    assert submitted_form_bad(admin_browser)



from django.db.models import Max

from bpp.models.multiseek import BppMultiseekVisibility


class BppMultiseekVisibilityMixin:
    """Klasa zapewniająca funkcję .enable, która to z kolei odpytuje bazę danych
    o dostępność lub niedostępność danego obiektu dla danego użytkownika."""

    def _get_or_create(self):
        field_name = self.field_name
        if (
            hasattr(self, "bpp_multiseek_visibility_field_name")
            and self.bpp_multiseek_visibility_field_name
        ):
            field_name = self.bpp_multiseek_visibility_field_name

        try:
            vis = BppMultiseekVisibility.objects.get(field_name=field_name)

            if vis.label != self.label:
                vis.label = self.label
                vis.save()

        except BppMultiseekVisibility.DoesNotExist:
            max_sort_order = BppMultiseekVisibility.objects.aggregate(Max("sort_order"))
            vis = BppMultiseekVisibility.objects.create(
                field_name=field_name,
                label=self.label,
                public=self.public,
                authenticated=True,
                staff=True,
                sort_order=(max_sort_order["sort_order__max"] or 0) + 1,
            )

        return vis

    @property
    def visibility_options(self):
        return self._get_or_create()

    def option_enabled(self):
        return True

    @property
    def ui_order(self):
        return self.visibility_options.sort_order

    def enabled(self, request=None):
        if not self.option_enabled():
            return False

        vis = self.visibility_options

        if request is None or not request.user.is_authenticated:
            # Jeżeli nie został podany parametr 'request' lub użytkownik jest anonimowy
            # to zwróc wartość parametru 'widoczny dla wszystkich'
            return vis.public

        if request.user.is_staff:
            return vis.staff

        if request.user.is_authenticated:
            return vis.authenticated

        raise NotImplementedError("This should not happen")



# Generated by Django 3.0.11 on 2021-03-07 10:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0241_auto_20210228_1916"),
    ]

    operations = [
        migrations.CreateModel(
            name="Grupa_Pracownicza",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("nazwa", models.CharField(max_length=512, unique=True)),
            ],
            options={
                "verbose_name": "grupa pracownicza",
                "verbose_name_plural": "grupy pracownicze",
                "ordering": ["nazwa"],
            },
        ),
        migrations.CreateModel(
            name="Wymiar_Etatu",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("nazwa", models.CharField(max_length=512, unique=True)),
            ],
            options={
                "verbose_name": "wymiar etatu",
                "verbose_name_plural": "wymiary etatów",
                "ordering": ["nazwa"],
            },
        ),
        migrations.AddField(
            model_name="autor_jednostka",
            name="grupa_pracownicza",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.Grupa_Pracownicza",
            ),
        ),
        migrations.AddField(
            model_name="autor_jednostka",
            name="wymiar_etatu",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.Wymiar_Etatu",
            ),
        ),
    ]



# Generated by Django 2.1.7 on 2019-07-07 18:44

import bpp.models.cache
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [('bpp', '0165_cache_punktacja_autora_cache_punktacja_dyscypliny'), ('bpp', '0166_auto_20190702_1200'), ('bpp', '0167_auto_20190707_2029')]

    dependencies = [
        ('bpp', '0164_auto_20190701_1303'),
    ]

    operations = [
        migrations.CreateModel(
            name='Cache_Punktacja_Autora',
            fields=[
                ('rekord_id', bpp.models.cache.TupleField(base_field=models.IntegerField(), db_index=True, size=2)),
                ('pkdaut', models.DecimalField(decimal_places=4, max_digits=20)),
                ('slot', models.DecimalField(decimal_places=4, max_digits=20)),
                ('autor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Autor')),
                ('dyscyplina', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Dyscyplina_Naukowa')),
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='Cache_Punktacja_Dyscypliny',
            fields=[
                ('rekord_id', bpp.models.cache.TupleField(base_field=models.IntegerField(), db_index=True, size=2)),
                ('pkd', models.DecimalField(decimal_places=4, max_digits=20)),
                ('slot', models.DecimalField(decimal_places=4, max_digits=20)),
                ('dyscyplina', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Dyscyplina_Naukowa')),
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
    ]



import os

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

import pytest
from webtest.forms import Upload


@pytest.mark.django_db
def test_serwer_testowy(client):
    # W przypadku, gdy domena zawiera słowo "test", to na stronie powinien
    # pojawić się obrazek tła oraz słowo "SERWER TESTOWY" w tytule
    response = client.get("/", SERVER_NAME="test.unexistenttld")
    assert b"<b>Serwer testowy" in response.content

    response = client.get("/", SERVER_NAME="127.0.0.1")
    assert b"<b>Serwer testowy" not in response.content


def test_views_main(admin_client):
    res = admin_client.get(reverse("integrator2:main"))
    assert "Brak plików" in res.rendered_content


def test_views_upload_lista_ministerialna(admin_app):
    page = admin_app.get(reverse("integrator2:upload_lista_ministerialna"))

    form = page.form
    form["file"] = Upload(os.path.dirname(__file__) + "/xls/lista_a_krotka.xlsx")
    res = form.submit().maybe_follow()

    assert "Plik został dodany" in res.text


def test_views_detail(admin_app, admin_user, lmi):
    lmi.owner = admin_user
    lmi.save()

    url = reverse("integrator2:detail", args=(lmi._meta.model_name, lmi.pk))
    page = admin_app.get(url)

    assert b"xlsx" in page.content



import random

import pytest

from bpp.tests import normalize_html

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from django.conf import settings
from model_bakery import baker

from fixtures import JEDNOSTKA_PODRZEDNA, JEDNOSTKA_UCZELNI

from bpp.models import Autor_Jednostka, Uczelnia
from bpp.models.autor import Autor
from bpp.views.browse import JednostkiView


@pytest.mark.django_db
def test_jednostka_nie_wyswietlaj_autorow_gdy_wielu(client, jednostka):
    for n in range(settings.MAX_NO_AUTHORS_ON_BROWSE_JEDNOSTKA_PAGE + 1):
        jednostka.dodaj_autora(baker.make(Autor))

    res = client.get(reverse("bpp:browse_jednostka", args=(jednostka.slug,)))
    assert "... napisane przez" not in res.rendered_content


def test_browse_jednostka_nadrzedna(jednostka, jednostka_podrzedna, client):
    url = reverse("bpp:browse_jednostka", args=(jednostka.slug,))
    page = client.get(url)
    assert "Jest nadrzędną jednostką dla" in normalize_html(page.rendered_content)

    url = reverse("bpp:browse_jednostka", args=(jednostka_podrzedna.slug,))
    page = client.get(url)
    assert "Wchodzi w skład" in normalize_html(page.rendered_content)


@pytest.mark.django_db
def test_browse_jednostka_paginate_by(uczelnia: Uczelnia):
    j = JednostkiView()
    assert j.get_paginate_by(None) == uczelnia.ilosc_jednostek_na_strone

    ile = random.randint(10, 10000)
    uczelnia.ilosc_jednostek_na_strone = ile
    uczelnia.save()
    assert j.get_paginate_by(None) == ile


def test_browse_jednostka_sortowanie(jednostka, jednostka_podrzedna, uczelnia, client):

    jednostka.nazwa = "Z jednostka"
    jednostka.save()

    jednostka_podrzedna.nazwa = "A jednostka"
    jednostka_podrzedna.save()

    uczelnia.sortuj_jednostki_alfabetycznie = True
    uczelnia.save()

    url = reverse("bpp:browse_jednostki")
    page = client.get(url)
    idx1 = page.rendered_content.find("A jednostka")
    idx2 = page.rendered_content.find("Z jednostka")

    assert idx1 < idx2

    uczelnia.sortuj_jednostki_alfabetycznie = False
    uczelnia.save()
    page = client.get(url)

    idx1 = page.rendered_content.find("A jednostka")
    idx2 = page.rendered_content.find("Z jednostka")
    assert idx1 > idx2


def test_browse_jednostka_nadrzedna_tekst(jednostka_podrzedna, jednostka, client):
    url = reverse("bpp:browse_jednostka", args=(jednostka.slug,))
    res = client.get(url)
    assert "Jest nadrzędną jednostką dla" in normalize_html(res.rendered_content)

    url = reverse("bpp:browse_jednostka", args=(jednostka_podrzedna.slug,))
    res = client.get(url)
    assert "Jest nadrzędną jednostką dla" not in normalize_html(res.rendered_content)


def test_browse_pokazuj_tylko_jednostki_nadrzedne_nie(
    jednostka_podrzedna, jednostka, client, uczelnia
):
    url = reverse("bpp:browse_jednostki")
    res = client.get(url)
    assert JEDNOSTKA_UCZELNI in normalize_html(res.rendered_content)
    assert JEDNOSTKA_PODRZEDNA in normalize_html(res.rendered_content)


def test_browse_pokazuj_tylko_jednostki_nadrzedne_tak(
    jednostka_podrzedna, jednostka, client, uczelnia
):
    uczelnia.pokazuj_tylko_jednostki_nadrzedne = True
    uczelnia.save()

    url = reverse("bpp:browse_jednostki")
    res = client.get(url)
    assert JEDNOSTKA_UCZELNI in normalize_html(res.rendered_content)
    assert JEDNOSTKA_PODRZEDNA not in normalize_html(res.rendered_content)


@pytest.mark.parametrize("arg_res", [True, False])
def test_jednostka_pokazuj_opis(jednostka, client, arg_res):
    TESTSTR = "opis=foobar"
    jednostka.pokazuj_opis = arg_res
    jednostka.opis = TESTSTR
    jednostka.save()

    url = reverse("bpp:browse_jednostka", args=(jednostka.slug,))
    res = client.get(url)
    result = TESTSTR in normalize_html(res.rendered_content)
    assert result is arg_res


def test_jednostka_aktualni_pracownicy(
    jednostka, autor_jan_nowak, autor_jan_kowalski, wydawnictwo_ciagle, client
):
    # Kowalski to obecny pracownik
    Autor_Jednostka.objects.create(
        autor=autor_jan_kowalski, jednostka=jednostka, podstawowe_miejsce_pracy=True
    )

    # Nowak to osoba ktora wczesniej miala publikacje
    wydawnictwo_ciagle.dodaj_autora(autor=autor_jan_nowak, jednostka=jednostka)

    url = reverse("bpp:browse_jednostka", args=(jednostka.slug,))
    res = client.get(url)
    html = normalize_html(res.rendered_content)
    assert "napisane przez obecnych pracowników" in html
    assert "osoby wcześniej związane" in html
    assert "napisane przez:" not in html



from django.urls import path

from import_pracownikow.views import (
    ImportPracownikowDetailsView,
    ImportPracownikowResetujPodstawoweMiejscePracyView,
    ImportPracownikowResultsView,
    ImportPracownikowRouterView,
    ListaImportowView,
    NowyImportView,
    RestartImportView,
)

app_name = "import_pracownikow"

urlpatterns = [
    path("", ListaImportowView.as_view(), name="index"),
    path("new/", NowyImportView.as_view(), name="new"),
    path(
        "<uuid:pk>/",
        ImportPracownikowRouterView.as_view(),
        name="importpracownikow-router",
    ),
    path(
        "<uuid:pk>/details/",
        ImportPracownikowDetailsView.as_view(),
        name="importpracownikow-details",
    ),
    path(
        "<uuid:pk>/results/",
        ImportPracownikowResultsView.as_view(),
        name="importpracownikow-results",
    ),
    path(
        "<uuid:pk>/resetuj-podstawowe-miejsce-pracy/",
        ImportPracownikowResetujPodstawoweMiejscePracyView.as_view(),
        name="importpracownikow-resetuj-podstawowe-miejsce-pracy",
    ),
    path(
        "<uuid:pk>/regen/",
        RestartImportView.as_view(),
        name="importpracownikow-restart",
    ),
]



from django.db import transaction
from django.urls import reverse

from import_dyscyplin.models import Import_Dyscyplin, Import_Dyscyplin_Row


def wyslij(wd_app, plik):
    assert Import_Dyscyplin.objects.all().count() == 0
    c = wd_app.get(reverse("import_dyscyplin:create")).maybe_follow()
    c.forms[0]["plik"].value = [
        plik,
    ]
    res = c.forms[0].submit().maybe_follow()
    assert Import_Dyscyplin.objects.all().count() == 1

    return res


def wyslij_i_przeanalizuj(wd_app, plik):
    wyslij(wd_app, plik)

    i = Import_Dyscyplin.objects.all().first()
    i.stworz_kolumny()

    if not i.stan == Import_Dyscyplin.STAN.BLEDNY:
        i.zatwierdz_kolumny()

        if not i.stan == Import_Dyscyplin.STAN.BLEDNY:
            i.przeanalizuj()

    i.save()
    return i


def test_CreateImport_DyscyplinView_bledny_plik(wd_app, conftest_py, transactional_db):
    with transaction.atomic():
        wyslij_i_przeanalizuj(wd_app, conftest_py)

    assert Import_Dyscyplin.objects.all().count() == 1
    i = Import_Dyscyplin.objects.all().first()
    assert i.bledny == True
    assert i.stan == Import_Dyscyplin.STAN.BLEDNY


def test_CreateImport_DyscyplinView_dobry_plik(wd_app, test1_xlsx, transactional_db):
    with transaction.atomic():
        wyslij_i_przeanalizuj(wd_app, test1_xlsx)

    assert Import_Dyscyplin.objects.all().count() == 1
    i = Import_Dyscyplin.objects.all().first()
    assert i.bledny == False
    assert i.stan == Import_Dyscyplin.STAN.PRZEANALIZOWANY


def test_ListImport_Dyscyplin(wd_app, test1_xlsx):
    wyslij_i_przeanalizuj(wd_app, test1_xlsx)
    res = wd_app.get(reverse("import_dyscyplin:index"))
    assert "test1" in res.testbody
    assert ".xlsx" in res.testbody


def test_UruchomPrzetwarzanieImport_Dyscyplin(wd_app, test1_xlsx):
    wyslij(wd_app, test1_xlsx)

    i = Import_Dyscyplin.objects.all().first()
    i.stworz_kolumny()
    i.zatwierdz_kolumny()
    i.save()

    g = wd_app.get(reverse("import_dyscyplin:przetwarzaj", args=(i.pk,)))
    assert g.json["status"] == "ok"


def test_UsunImport_Dyscyplin(wd_app, test1_xlsx, transactional_db):
    wyslij(wd_app, test1_xlsx)
    assert Import_Dyscyplin.objects.all().count() == 1
    i = Import_Dyscyplin.objects.all().first()
    g = wd_app.get(reverse("import_dyscyplin:usun", args=(i.pk,))).maybe_follow()
    assert "Brak informacji o importowanych" in g.testbody
    assert Import_Dyscyplin.objects.all().count() == 0


def test_API_Do_IntegracjiView(wd_app, test1_xlsx, autor_jan_nowak):
    i = wyslij_i_przeanalizuj(wd_app, test1_xlsx)
    i.integruj_dyscypliny()

    r = Import_Dyscyplin_Row.objects.all().first()
    r.autor = autor_jan_nowak
    r.stan = Import_Dyscyplin_Row.STAN.NOWY
    r.save()

    res = wd_app.get(reverse("import_dyscyplin:api_do_integracji", args=(i.pk,)))
    assert len(res.json["data"]) == 1
    assert res.json["data"][0]["nazwisko"] == "Kowalski"


def test_API_Nie_Do_IntegracjiView(wd_app, test1_xlsx):
    i = wyslij_i_przeanalizuj(wd_app, test1_xlsx)

    res = wd_app.get(reverse("import_dyscyplin:api_nie_do_integracji", args=(i.pk,)))
    assert len(res.json["data"]) == 6


def test_API_Zintegrowane(wd_app, test1_xlsx):
    i = wyslij_i_przeanalizuj(wd_app, test1_xlsx)

    r = Import_Dyscyplin_Row.objects.all().first()
    r.stan = Import_Dyscyplin_Row.STAN.ZINTEGROWANY
    r.save()

    res = wd_app.get(reverse("import_dyscyplin:api_zintegrowane", args=(i.pk,)))
    assert len(res.json["data"]) == 1


def test_UruchomIntegracjeImport_DyscyplinView(
    wd_app, wprowadzanie_danych_user, test1_xlsx
):
    i = wyslij_i_przeanalizuj(wd_app, test1_xlsx)
    i.integruj_dyscypliny()
    i.owner = wprowadzanie_danych_user
    i.save()

    res = wd_app.get(reverse("import_dyscyplin:integruj", args=(i.pk,)))
    assert res.json["status"] == "ok"



# Generated by Django 3.0.11 on 2021-04-27 19:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0262_bppuser_pbn_token"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_aktualizuj_na_biezaco",
            field=models.BooleanField(
                default=False,
                help_text="Aktualizuj rekordy w PBN przy każdym zapisie rekordu",
                verbose_name="Aktualizuj PBN na bieżąco",
            ),
        ),
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_integracja",
            field=models.BooleanField(
                default=False,
                help_text="Wymaga prawidłowo wypełnionych pól 'Adres API w PBN', 'Nazwa aplikacji w PBN', 'Token aplikacji w PBN' oraz łączności między serwerem BPP a serwerem PBN. ",
                verbose_name="Czy używać integracji z PBN?",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-19 10:20
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion

def dodaj(apps, schema_editor):
    eksport_pbn_TYP_KBN_MAP = {
        'PO': 'original-article',
        'PW': 'original-article',
        'PP': 'review-article',
        'PNP': 'popular-science-article',
        '000': 'others-citable'
    }
    Typ_KBN = apps.get_model("bpp", "Typ_KBN")
    Charakter_PBN = apps.get_model("bpp", "Charakter_PBN")
    for key, value in eksport_pbn_TYP_KBN_MAP.items():
        t = Typ_KBN.objects.get(skrot=key)
        cp = Charakter_PBN.objects.get(identyfikator=value)
        t.charakter_pbn = cp
        t.save()


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0090_pbn_id_dla_prac'),
    ]

    operations = [
        migrations.AddField(
            model_name='typ_kbn',
            name='charakter_pbn',
            field=models.ForeignKey(blank=True, default=None, help_text='Wartość wybrana w tym polu zostanie użyta jako \n        fallback, tzn. jeżeli dla charakteru formalnego danego rekordu nie \n        określono odpowiedniego charakteru PBN, to zostanie użyta wartość \n        tego pola, o ile wybrana. ', null=True, on_delete=django.db.models.deletion.CASCADE, to='bpp.Charakter_PBN', verbose_name='Charakter PBN'),
        ),
        migrations.RunPython(
            dodaj,
            migrations.RunPython.noop
        )
    ]



# Lokalny development (na moim Maku)

import os


def setenv_default(varname, default_value):
    if os.environ.get(varname) is None:
        os.environ[varname] = default_value


setenv_default("DJANGO_SETTINGS_MODULE", "django_bpp.settings.local")
setenv_default("DJANGO_BPP_SECRET_KEY", "0xdeadbeef 2")

from .base import *  # noqa
from .base import DATABASES, INSTALLED_APPS, MIDDLEWARE

DEBUG = True

SENDFILE_BACKEND = "sendfile.backends.simple"

SELENIUM_DRIVER = "Firefox"

SESSION_COOKIE_SECURE = False
CSRF_COOKIE_SECURE = False

MEDIA_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "..", "..", "media")
)
SENDFILE_ROOT = MEDIA_ROOT

COMPRESS_ENABLED = False  # not DEBUG
COMPRESS_OFFLINE = False

ALLOWED_HOSTS = [
    "dockerhost",
    "webserver",
    "localhost",
    "127.0.0.1",
    "mac.iplweb",
    "test.unexistenttld",
]

HTML_MINIFY = False

CELERY_ALWAYS_EAGER = False  # True  # False
CELERY_EAGER_PROPAGATES_EXCEPTIONS = True

PUNKTUJ_MONOGRAFIE = False

DEBUG_TOOLBAR = False

if DEBUG_TOOLBAR and DEBUG:
    MIDDLEWARE = [
        "debug_toolbar.middleware.DebugToolbarMiddleware",
        "bpp.middleware.NonHtmlDebugToolbarMiddleware",
    ] + MIDDLEWARE

    INSTALLED_APPS.append("debug_toolbar")

DATABASES["default"]["CONN_MAX_AGE"] = 0


# Vide komentarz w TEMPLATES[0]["OPTIONS"]["loaders"]
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    }
}


TEMPLATES[0]["OPTIONS"]["loaders"] = [  # noqa
    "admin_tools.template_loaders.Loader",
    "dbtemplates.loader.Loader",
    "django.template.loaders.filesystem.Loader",
    "django.template.loaders.app_directories.Loader",
]

PASSWORD_HASHERS = ["django.contrib.auth.hashers.MD5PasswordHasher"]



# Generated by Django 3.0.14 on 2021-08-20 13:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0030_auto_20210818_2343"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scientist",
            name="from_institution_api",
            field=models.NullBooleanField(
                db_index=True, verbose_name="Rekord z API instytucji"
            ),
        ),
    ]



# -*- encoding: utf-8 -*-
import sys
from decimal import Decimal

from django.core.management import BaseCommand
from django.db import transaction

import logging

from bpp.models import Autor, Rekord, Cache_Punktacja_Autora

logger = logging.getLogger("django")


class Command(BaseCommand):
    help = "Zbiera sloty dla danego autora"

    def add_arguments(self, parser):
        parser.add_argument("autor_id", type=int, help="ID autora")
        parser.add_argument("--slot", type=int, default=4, help="Ile slotow zbierac")
        parser.add_argument("--xls", default=None)

        parser.add_argument(
            "--rok_min", type=int, default=2017,
        )
        parser.add_argument(
            "--rok_max", type=int, default=2020,
        )

    @transaction.atomic
    def handle(
        self, autor_id, slot, rok_min, rok_max, verbosity, xls, *args, **options
    ):
        autor = Autor.objects.get(id=autor_id)

        res, lista = autor.zbieraj_sloty(slot, rok_min, rok_max)

        wiersze = []
        wiersze.append(["Parametry:"])
        wiersze.append(["Autor", str(autor)])
        wiersze.append(["Rok min", str(rok_min)])
        wiersze.append(["Rok max", str(rok_max)])
        wiersze.append(["Zbierac do: ", str(slot)])
        wiersze.append([])
        wiersze.append(["Zebrano punktow", str(res)])
        wiersze.append([])

        prace = []
        prace.append(["Prace:"])
        prace.append(["Rok", "ID", "Slot", "PKdAut", "Tytuł (skrócony)"])
        suma_slotow = Decimal("0.0000")
        for elem in lista:
            r = Rekord.objects.get(pk=elem)
            cpa = Cache_Punktacja_Autora.objects.get(rekord_id=r.pk, autor=autor)
            suma_slotow += cpa.slot
            prace.append(
                [
                    str(r.rok),
                    str(r.pk),
                    str(cpa.slot).replace(".", ","),
                    str(cpa.pkdaut).replace(".", ","),
                    str(r.tytul_oryginalny[:80]),
                ]
            )

        wiersze.append(["Zebrano slot(ow):", str(suma_slotow)])
        wiersze.append([])

        if not xls:
            for w in wiersze + prace:
                print("\t".join(w))
            sys.exit(0)

        import openpyxl

        wb = openpyxl.Workbook()
        s = wb.worksheets[0]
        for w in wiersze + prace:
            s.append(w)
        wb.save(xls)



# Create your models here.
from copy import copy
from datetime import date, timedelta

from django import forms
from django.core.serializers.json import DjangoJSONEncoder
from django.db import DataError, IntegrityError, models, transaction
from django.db.models import JSONField, Q

from import_common.core import (
    matchuj_autora,
    matchuj_funkcja_autora,
    matchuj_grupa_pracownicza,
    matchuj_jednostke,
    matchuj_wymiar_etatu,
)
from import_common.exceptions import (
    BPPDatabaseError,
    BPPDatabaseMismatch,
    XLSMatchError,
    XLSParseError,
)
from import_common.forms import ExcelDateField
from import_common.models import ImportRowMixin
from import_common.normalization import (
    normalize_funkcja_autora,
    normalize_grupa_pracownicza,
    normalize_nullboleanfield,
    normalize_wymiar_etatu,
)
from import_common.util import XLSImportFile
from long_running.models import Operation
from long_running.notification_mixins import ASGINotificationMixin

from django.utils import timezone

from bpp.models import (
    Autor,
    Autor_Jednostka,
    Funkcja_Autora,
    Grupa_Pracownicza,
    Jednostka,
    Tytul,
    Wymiar_Etatu,
)


class JednostkaForm(forms.Form):
    nazwa_jednostki = forms.CharField(max_length=10240)
    wydział = forms.CharField(max_length=500)


class AutorForm(forms.Form):
    nazwisko = forms.CharField(max_length=200)
    imię = forms.CharField(max_length=200)

    numer = forms.IntegerField(required=False)
    orcid = forms.CharField(max_length=19, required=False)
    tytuł_stopień = forms.CharField(max_length=200, required=False)
    pbn_uuid = forms.UUIDField(required=False)
    bpp_id = forms.IntegerField(required=False)

    stanowisko = forms.CharField(max_length=200)
    grupa_pracownicza = forms.CharField(max_length=200)
    data_zatrudnienia = ExcelDateField()
    data_końca_zatrudnienia = ExcelDateField(required=False)
    podstawowe_miejsce_pracy = forms.BooleanField(required=False)
    wymiar_etatu = forms.CharField(max_length=200)


class ImportPracownikow(ASGINotificationMixin, Operation):
    plik_xls = models.FileField()

    performed = models.BooleanField(default=False)
    integrated = models.BooleanField(default=False)

    @transaction.atomic
    def on_reset(self):
        self.performed = self.integrated = False
        self.importpracownikowrow_set.all().delete()
        self.save()

    def perform(self):
        xif = XLSImportFile(self.plik_xls.path)
        total = xif.count()

        for no, elem in enumerate(xif.data()):

            jednostka_form = JednostkaForm(data=elem)
            jednostka_form.full_clean()
            if not jednostka_form.is_valid():
                raise XLSParseError(elem, jednostka_form, "weryfikacja nazwy jednostki")

            try:
                jednostka = matchuj_jednostke(
                    jednostka_form.cleaned_data.get("nazwa_jednostki"),
                    wydzial=jednostka_form.cleaned_data.get("wydział"),
                )
            except Jednostka.MultipleObjectsReturned:
                raise XLSMatchError(
                    elem, "jednostka", "wiele dopasowań w systemie - po nazwie"
                )

            except Jednostka.DoesNotExist:
                raise XLSMatchError(
                    elem, "jednostka", "brak dopasowania w systemie - po nazwie"
                )

            autor_form = AutorForm(data=elem)
            autor_form.full_clean()
            if not autor_form.is_valid():
                raise XLSParseError(elem, autor_form, "weryfikacja danych autora")
            assert isinstance(autor_form.cleaned_data.get("data_zatrudnienia"), date)
            data = autor_form.cleaned_data

            # if data.get("tytuł_stopień"):
            #     try:
            #         matchuj_tytul(data.get("tytuł_stopień"))
            #     except Tytul.DoesNotExist:
            #         raise XLSMatchError(
            #             elem, "tytuł", "brak takiego tytułu w systemie (nazwa, skrót)"
            #         )
            #     except Tytul.MultipleObjectsReturned:
            #         raise XLSMatchError(
            #             elem,
            #             "tytuł",
            #             "liczne dopasowania dla takiego tytułu w systemie",
            #         )

            try:
                funkcja_autora = matchuj_funkcja_autora(data.get("stanowisko"))
            except Funkcja_Autora.DoesNotExist:
                try:
                    funkcja_autora = Funkcja_Autora.objects.create(
                        nazwa=normalize_funkcja_autora(data.get("stanowisko")),
                        skrot=normalize_funkcja_autora(data.get("stanowisko")),
                    )
                except IntegrityError:
                    raise XLSParseError(
                        elem,
                        autor_form,
                        "nie można utworzyć nowego stanowiska na bazie takich danych",
                    )

            except Funkcja_Autora.MultipleObjectsReturned:
                raise XLSMatchError(
                    elem,
                    "stanowisko",
                    "liczne dopasowania dla takiej funkcji autora (stanowiska) w systemie",
                )

            try:
                grupa_pracownicza = matchuj_grupa_pracownicza(
                    data.get("grupa_pracownicza")
                )
            except Grupa_Pracownicza.DoesNotExist:
                grupa_pracownicza = Grupa_Pracownicza.objects.create(
                    nazwa=normalize_grupa_pracownicza(data.get("grupa_pracownicza"))
                )

            try:
                wymiar_etatu = matchuj_wymiar_etatu(data.get("wymiar_etatu"))
            except Wymiar_Etatu.DoesNotExist:
                wymiar_etatu = Wymiar_Etatu.objects.create(
                    nazwa=normalize_wymiar_etatu(data.get("wymiar_etatu"))
                )

            tytul_str = data.get("tytuł_stopień")

            autor = matchuj_autora(  # noqa
                imiona=data.get("imię"),
                nazwisko=data.get("nazwisko"),
                jednostka=jednostka,
                bpp_id=data.get("bpp_id"),
                pbn_uid_id=data.get("pbn_uuid"),
                system_kadrowy_id=data.get("numer"),
                pbn_id=data.get("pbn_id"),
                orcid=data.get("orcid"),
                tytul_str=tytul_str,
            )
            if autor is None:
                raise XLSMatchError(
                    elem, "autor", "brak dopasowania - różne kombinacje"
                )

            # Jeżeli w danych jest podane BPP ID, ale zwrócony autor nie zmatchował po podanym BPP ID
            # za to zmatchował po innych danych, sprawdźmy czy zwrócone BPP ID jest identyczne
            if data.get("bpp_id") is not None:
                if data.get("bpp_id") != autor.pk:
                    raise XLSMatchError(
                        elem,
                        "autor",
                        "BPP ID zmatchowanego autora i BPP ID w pliku XLS nie zgadzają się",
                    )

            try:
                aj = Autor_Jednostka.objects.get(autor=autor, jednostka=jednostka)
            except Autor_Jednostka.MultipleObjectsReturned:
                if "data_zatrudnienia" in data:
                    try:
                        aj = Autor_Jednostka.objects.get(
                            autor=autor,
                            jednostka=jednostka,
                            rozpoczal_prace=data.get("data_zatrudnienia"),
                        )
                    except Autor_Jednostka.DoesNotExist:
                        raise BPPDatabaseMismatch(
                            elem,
                            "autor + jednostka",
                            "brak jednoznacznego powiązania autor+jednostka po stronie BPP",
                        )
            except Autor_Jednostka.DoesNotExist:
                aj = Autor_Jednostka.objects.create(
                    autor=autor, jednostka=jednostka, funkcja=funkcja_autora
                )

            tytul = None
            try:
                if tytul_str:
                    tytul = Tytul.objects.get(Q(nazwa=tytul_str) | Q(skrot=tytul_str))
            except Tytul.DoesNotExist:
                pass

            res = ImportPracownikowRow(
                parent=self,
                dane_z_xls=elem,
                dane_znormalizowane=copy(autor_form.cleaned_data),
                autor=autor,
                jednostka=jednostka,
                autor_jednostka=aj,
                tytul=tytul,
                funkcja_autora=funkcja_autora,
                grupa_pracownicza=grupa_pracownicza,
                wymiar_etatu=wymiar_etatu,
                podstawowe_miejsce_pracy=normalize_nullboleanfield(
                    elem.get("podstawowe_miejsce_pracy")
                ),
            )
            res.zmiany_potrzebne = res.check_if_integration_needed()
            res.save()

            if no % 10 == 0:
                self.send_progress(no / total / 2.0)

        self.performed = True
        self.save()

        self.integrate()
        self.integrated = True
        self.save()

    @property
    def zmiany_potrzebne_set(self):
        return self.importpracownikowrow_set.filter(zmiany_potrzebne=True)

    def get_details_set(self):
        return self.importpracownikowrow_set.all().select_related(
            "autor",
            "jednostka",
            "jednostka__wydzial",
            "autor__tytul",
            "grupa_pracownicza",
            "funkcja_autora",
            "wymiar_etatu",
        )

    def autorzy_spoza_pliku_set(self, uczelnia=None, today=None):
        """
        Zwraca wszystkie połączenia Autor + Jednostka, gdzie:
        1) połączenie autor + jednostka nie występuje w imporcie danych (self)
        2) jednostka nie jest obca,
        3) jednostka ma pole "zarzadzaj_automatycznie" zaznaczone jako True
        """

        if today is None:
            today = timezone.now().date()

        autorzy_jednostki_z_pliku = self.importpracownikowrow_set.values_list(
            "autor_jednostka"
        ).distinct()

        qry = (
            Autor_Jednostka.objects.exclude(pk__in=autorzy_jednostki_z_pliku)
            .exclude(autor__aktualna_jednostka=None)
            .exclude(jednostka__zarzadzaj_automatycznie=False)
            .exclude(zakonczyl_prace__lte=today)
        )

        if uczelnia is not None and uczelnia.obca_jednostka_id is not None:
            qry = qry.exclude(autor__aktualna_jednostka_id=uczelnia.obca_jednostka_id)

        return qry

    @transaction.atomic
    def odepnij_autorow_spoza_pliku(self, uczelnia=None, today=None, yesterday=None):
        if today is None:
            today = timezone.now().date()

        if yesterday is None:
            yesterday = today - timedelta(days=1)

        for elem in self.autorzy_spoza_pliku_set(uczelnia=uczelnia, today=today):
            elem.zakonczyl_prace = yesterday
            elem.podstawowe_miejsce_pracy = False
            elem.save()

            elem.refresh_from_db()

    def on_finished(self):
        self.send_processing_finished()

    def integrate(self):
        total = self.zmiany_potrzebne_set.all().count()
        for no, elem in enumerate(
            self.zmiany_potrzebne_set.all().select_related(
                "autor", "jednostka", "jednostka__wydzial", "autor__tytul"
            )
        ):
            elem.integrate()
            self.send_progress(0.5 + (no / total / 2.0))


class ImportPracownikowRow(ImportRowMixin, models.Model):
    parent = models.ForeignKey(
        ImportPracownikow,
        on_delete=models.CASCADE,  # related_name="row_set"
    )
    dane_z_xls = JSONField(null=True, blank=True, encoder=DjangoJSONEncoder)
    dane_znormalizowane = JSONField(null=True, blank=True, encoder=DjangoJSONEncoder)

    autor = models.ForeignKey(Autor, on_delete=models.CASCADE)
    jednostka = models.ForeignKey(Jednostka, on_delete=models.CASCADE)
    autor_jednostka = models.ForeignKey(Autor_Jednostka, on_delete=models.CASCADE)

    podstawowe_miejsce_pracy = models.BooleanField(null=True, blank=True, default=None)
    funkcja_autora = models.ForeignKey(Funkcja_Autora, on_delete=models.CASCADE)
    grupa_pracownicza = models.ForeignKey(Grupa_Pracownicza, on_delete=models.CASCADE)
    wymiar_etatu = models.ForeignKey(Wymiar_Etatu, on_delete=models.CASCADE)
    tytul = models.ForeignKey(Tytul, on_delete=models.SET_NULL, null=True)

    zmiany_potrzebne = models.BooleanField()

    log_zmian = JSONField(encoder=DjangoJSONEncoder, null=True, blank=True)

    MAPPING_DANE_NA_AUTOR = [
        ("numer", "system_kadrowy_id"),
        ("orcid", "orcid"),
        ("pbn_uuid", "pbn_uuid"),
    ]

    @property
    def dane_bardziej_znormalizowane(self):
        """parsuje daty w dwóch polach, bo JSON w PostgreSQL to raz, a JSONDecoder
        w Django nie ma czegos takiego jak dekoder JSON do pól JSON"""
        for fld in ["data_zatrudnienia", "data_końca_zatrudnienia"]:
            if self.dane_znormalizowane.get(fld):
                v = self.dane_znormalizowane.get(fld)
                if v is None or isinstance(v, date) or v == "":
                    continue
                self.dane_znormalizowane[fld] = date.fromisoformat(v)

        return self.dane_znormalizowane

    def check_if_integration_needed(self):
        dane = self.dane_bardziej_znormalizowane

        # aktualizacja Autora
        a = self.autor

        def _spr(klucz_danych, atrybut_autora):
            v = dane.get(klucz_danych)
            if v is None or str(v) == "":
                return

            if getattr(a, atrybut_autora) != v:
                return True

        for klucz_danych, atrybut_autora in self.MAPPING_DANE_NA_AUTOR:
            if _spr(klucz_danych, atrybut_autora):
                return True

        # aktualizacja Autor_Jednostka
        aj = self.autor_jednostka

        if (
            dane.get("data_zatrudnienia") is not None
            and aj.rozpoczal_prace != dane["data_zatrudnienia"]
        ):
            return True

        if (
            dane.get("data_końca_zatrudnienia") is not None
            and aj.zakonczyl_prace != dane["data_końca_zatrudnienia"]
        ):
            return True

        if aj.funkcja != self.funkcja_autora:
            return True

        if aj.grupa_pracownicza != self.grupa_pracownicza:
            return True

        if aj.wymiar_etatu != self.wymiar_etatu:
            return True

        if self.podstawowe_miejsce_pracy != aj.podstawowe_miejsce_pracy:
            return True

        if self.tytul_id != a.tytul_id:
            return True

        return False

    def _integrate_autor(self):
        dane = self.dane_znormalizowane
        a = self.autor

        def _spr(klucz_danych, atrybut_autora):
            v = dane.get(klucz_danych)
            if v is None or (str(v) == ""):
                return

            if getattr(a, atrybut_autora) != v:
                return True

        for klucz_danych, atrybut_autora in self.MAPPING_DANE_NA_AUTOR:
            if _spr(klucz_danych, atrybut_autora):
                self.log_zmian["autor"].append(
                    f"{atrybut_autora} -> {dane.get(klucz_danych)}"
                )
                setattr(a, atrybut_autora, dane.get(klucz_danych))

        if self.tytul_id is not None:
            if a.tytul_id != self.tytul_id:
                a.tytul_id = self.tytul_id
                self.log_zmian["autor"].append(
                    f"tytuł naukowy -> {self.tytul.skrot if self.tytul_id else 'brak'}"
                )

        try:
            a.save()
        except DataError as e:
            raise BPPDatabaseError(self.dane_z_xls, self, f"DataError {e}")

    def _integrate_autor_jednostka(self):
        aj = self.autor_jednostka
        dane = self.dane_bardziej_znormalizowane

        if (
            dane.get("data_zatrudnienia") is not None
            and aj.rozpoczal_prace != dane["data_zatrudnienia"]
        ):
            aj.rozpoczal_prace = dane["data_zatrudnienia"]
            self.log_zmian["autor_jednostka"].append(
                f"data zatrudnienia na {dane['data_zatrudnienia']}"
            )

        if (
            dane.get("data_końca_zatrudnienia") is not None
            and aj.zakonczyl_prace != dane["data_końca_zatrudnienia"]
        ):
            aj.zakonczyl_prace = dane["data_końca_zatrudnienia"]
            self.log_zmian["autor_jednostka"].append(
                f"data końca zatrudnienia na {dane['data_końca_zatrudnienia']}"
            )

        if aj.funkcja != self.funkcja_autora:
            aj.funkcja = self.funkcja_autora
            self.log_zmian["autor_jednostka"].append(
                f"funkcja na {self.funkcja_autora}"
            )

        if aj.grupa_pracownicza != self.grupa_pracownicza:
            aj.grupa_pracownicza = self.grupa_pracownicza
            self.log_zmian["autor_jednostka"].append(
                f"grupa_pracownicza na {self.grupa_pracownicza}"
            )

        if aj.wymiar_etatu != self.wymiar_etatu:
            aj.wymiar_etatu = self.wymiar_etatu
            self.log_zmian["autor_jednostka"].append(
                f"wymiar_etatu na {self.wymiar_etatu}"
            )

        if self.podstawowe_miejsce_pracy != aj.podstawowe_miejsce_pracy:

            if not self.podstawowe_miejsce_pracy:
                aj.podstawowe_miejsce_pracy = False
                self.log_zmian["autor_jednostka"].append(
                    "podstawowe_miejsce_pracy -> nie"
                )
            else:
                aj.ustaw_podstawowe_miejsce_pracy()
                self.log_zmian["autor_jednostka"].append(
                    "podstawowe_miejsce_pracy -> tak"
                )

        aj.save()

    @transaction.atomic
    def integrate(self):
        assert self.zmiany_potrzebne
        self.log_zmian = {"autor": [], "autor_jednostka": []}
        self._integrate_autor()
        self._integrate_autor_jednostka()
        self.save()

    def sformatowany_log_zmian(self):
        if self.log_zmian is None:
            return

        if self.log_zmian["autor"]:
            yield "Zmiany obiektu Autor: " + ", ".join(
                [elem for elem in self.log_zmian["autor"]]
            )

        if self.log_zmian["autor_jednostka"]:
            yield "Zmiany obiektu Autor_Jednostka: " + ", ".join(
                [elem for elem in self.log_zmian["autor_jednostka"]]
            )

        if not self.log_zmian:
            return "bez zmian!"



from bpp.views import update_multiseek_title


def test_update_multiseek_title(rf):
    req = rf.post("/", data={"value": "test\r\nof\nfun"})
    req.session = {}
    res = update_multiseek_title(req)
    assert res.content == b'"test<br>of<br>fun"'



from braces.views import GroupRequiredMixin
from django.db import transaction
from django.views.generic.edit import FormView
from django.views.generic.list import ListView

from integrator2.forms import FormListaMinisterialna
from integrator2.models.lista_ministerialna import ListaMinisterialnaIntegration
from integrator2.tasks import analyze_file

from django.contrib import messages

from django.utils.functional import cached_property

from bpp.const import GR_WPROWADZANIE_DANYCH


class Main(GroupRequiredMixin, ListView):
    paginate_by = 10
    group_required = GR_WPROWADZANIE_DANYCH
    template_name = "main.html"

    def get_queryset(self):
        return ListaMinisterialnaIntegration.objects.filter(
            owner=self.request.user
        ).order_by("-uploaded_on")


class UploadListaMinisterialna(GroupRequiredMixin, FormView):
    group_required = GR_WPROWADZANIE_DANYCH
    template_name = "new.html"
    form_class = FormListaMinisterialna
    success_url = ".."

    def form_valid(self, form):
        form.instance.owner = self.request.user
        self.object = form.save()
        messages.add_message(
            self.request, messages.INFO, "Plik został dodany do kolejki przetwarzania."
        )
        transaction.on_commit(lambda: analyze_file.delay(self.object.pk))
        return super(FormView, self).form_valid(form)


class DetailBase(GroupRequiredMixin, ListView):
    # def get_template_names(self):
    #     return [self.kwargs['model_name'] + "_detail.html"]

    paginate_by = 100

    group_required = GR_WPROWADZANIE_DANYCH

    @cached_property
    def object(self):
        return self.get_object()

    def get_object(self):
        from django.apps import apps

        model = apps.get_model(
            app_label="integrator2", model_name=self.kwargs["model_name"]
        )
        return model.objects.get(pk=self.kwargs["pk"], owner=self.request.user)

    def get_queryset(self):
        return self.object.not_integrated().order_by("nazwa")

    def get_context_data(self, **kwargs):
        return super().get_context_data(object=self.object, **kwargs)



# Generated by Django 3.2.14 on 2022-08-07 21:29

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zglos_publikacje", "0004_auto_20220801_2128"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="zgloszenie_publikacji_autor",
            options={
                "verbose_name": "autor w zgłoszeniu publikacji",
                "verbose_name_plural": "autorzy w zgłoszeniu publikacji",
            },
        ),
        migrations.CreateModel(
            name="Zgloszenie_Publikacji_Plik",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "plik",
                    models.FileField(upload_to="", verbose_name="Plik załącznika"),
                ),
                (
                    "rekord",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="zglos_publikacje.zgloszenie_publikacji",
                    ),
                ),
            ],
            options={
                "verbose_name": "plik zgłoszenia publikacji",
                "verbose_name_plural": "pliki zgłoszeń publikacji",
            },
        ),
    ]



# -*- encoding: utf-8 -*-
import argparse
import logging
import multiprocessing
from math import floor

import django
from django.core.management import BaseCommand

import miniblog
from bpp.util import partition_count, disable_multithreading_by_monkeypatching_pool
from import_dbf.models import B_A, Bib
from import_dbf.util import (
    integruj_autorow,
    integruj_b_a,
    integruj_charaktery,
    integruj_funkcje_autorow,
    integruj_jednostki,
    integruj_jezyki,
    integruj_kbn,
    integruj_publikacje,
    integruj_tytuly_autorow,
    integruj_uczelnia,
    integruj_wydzialy,
    integruj_zrodla,
    mapuj_elementy_publikacji,
    przypisz_jednostki,
    sprawdz_zamapowanie_autorow,
    usun_podwojne_przypisania_b_a,
    wyswietl_prace_bez_dopasowania,
    wzbogacaj_charaktery,
    zatwierdz_podwojne_przypisania,
    dodaj_aktualnosc,
    set_sequences,
    przypisz_grupy_punktowe,
    utworz_szkielety_ksiazek,
    integruj_dyscypliny,
)

django.setup()


class Command(BaseCommand):
    help = "Integruje zaimportowaną bazę DBF z bazą BPP"

    def add_arguments(self, parser):
        parser.add_argument("--uczelnia", type=str, default="Domyślna Uczelnia")
        parser.add_argument("--skrot", type=str, default="DU")

        parser.add_argument("--disable-multithreading", action="store_true")

        parser.add_argument("--enable-all", action="store_true")
        parser.add_argument("--disable-transaction", action="store_true")

        parser.add_argument("--enable-wydzial", action="store_true")
        parser.add_argument("--enable-jednostka", action="store_true")
        parser.add_argument("--enable-autor", action="store_true")
        parser.add_argument("--enable-dyscypliny", action="store_true")
        parser.add_argument("--enable-publikacja", action="store_true")
        parser.add_argument("--enable-grupy-punktowe", action="store_true")
        parser.add_argument("--enable-szkielety-ksiazek", action="store_true")

        parser.add_argument("--enable-mapuj-publikacja", action="store_true")
        parser.add_argument("--enable-charakter-kbn-jezyk", action="store_true")
        parser.add_argument(
            "--charaktery-enrichment-xls", type=argparse.FileType("rb"), nargs="+"
        )

        parser.add_argument("--enable-zrodlo", action="store_true")
        parser.add_argument("--enable-b-a", action="store_true")
        parser.add_argument(
            "--enable-zatwierdz-podwojne-przypisania",
            action="store_true",
            help="""W przypadku, gdyby podwójne przypisania w bazie danych były OK, podaj ten argument
            aby utworzyć dodatkowe rekordy dla prawidłowo zdublowanych autorów""",
        )
        parser.add_argument("--enable-przypisz-jednostki", action="store_true")
        parser.add_argument("--enable-dodaj-aktualnosc", action="store_true")

    def handle(
        self,
        uczelnia,
        skrot,
        enable_all,
        disable_transaction,
        disable_multithreading,
        *args,
        **options
    ):
        verbosity = int(options["verbosity"])
        logger = logging.getLogger("django")
        if verbosity > 1:
            logger.setLevel(logging.DEBUG)

        from django import db

        db.connections.close_all()

        cpu_count = multiprocessing.cpu_count()
        num_proc = int(floor(cpu_count * 0.875)) or 1
        pool = multiprocessing.Pool(processes=num_proc)

        if disable_multithreading:
            disable_multithreading_by_monkeypatching_pool(pool)

        pool.apply(integruj_uczelnia, (uczelnia, skrot))

        if enable_all or options["enable_wydzial"]:
            logger.debug("Wydzialy")
            pool.apply(integruj_wydzialy)

        if enable_all or options["enable_jednostka"]:
            logger.debug("Jednostki")
            pool.apply(integruj_jednostki)

        if enable_all or options["enable_autor"]:
            pool.apply(integruj_tytuly_autorow)
            pool.apply(integruj_funkcje_autorow)

            # 'Jose Miguel', 'Caldas' <==> 'Jose', 'Miguel Caldas'
            # with fuj as (select imiona || ' ' || nazwisko as x, idt_aut as y
            # from import_dbf_aut where exp_id = idt_aut) select x, array_agg(y)
            # from fuj group by x having count(*) > 1

            logger.debug("Autorzy")
            pool.apply(integruj_autorow)
            logger.debug("Sprawdzam czy wszyscy sa przypisani")
            pool.apply(sprawdz_zamapowanie_autorow)

        if enable_all or options["enable_dyscypliny"]:
            pool.apply(integruj_dyscypliny)

        if enable_all or options["enable_charakter_kbn_jezyk"]:
            pool.apply(integruj_charaktery)

            fp = options.get("charaktery_enrichment_xls")
            if fp:
                pool.apply(wzbogacaj_charaktery, args=(fp[0].name,))

            pool.apply(integruj_kbn)
            pool.apply(integruj_jezyki)

        if enable_all or options["enable_zrodlo"]:
            logger.debug("Zrodla")
            pool.apply(integruj_zrodla)

        if enable_all or options["enable_mapuj_publikacja"]:
            logger.debug("Publikacje - wyciągam dane")

            pool.starmap(
                mapuj_elementy_publikacji,
                partition_count(Bib.objects.exclude(analyzed=True), num_proc),
            )

        if enable_all or options["enable_publikacja"]:
            logger.info("Integruje publikacje")
            pool.starmap(
                integruj_publikacje,
                partition_count(
                    Bib.objects.filter(object_id=None, analyzed=True), num_proc
                ),
            )

            pool.apply(wyswietl_prace_bez_dopasowania, (logger,))
            pool.apply(set_sequences)

        if enable_all or options["enable_grupy_punktowe"]:
            pool.apply(przypisz_grupy_punktowe)

        if enable_all or options["enable_szkielety_ksiazek"]:
            pool.apply(utworz_szkielety_ksiazek, (logger,))

        if enable_all or options["enable_zatwierdz_podwojne_przypisania"]:
            logger.debug("Zatwierdzanie podwójnych podwojnych przypisan")
            pool.apply(zatwierdz_podwojne_przypisania, (logger,))

        if enable_all or options["enable_b_a"]:
            logger.debug("Usuwanie podwojnych przypisan")
            pool.apply(usun_podwojne_przypisania_b_a, (logger,))
            logger.debug("Integracja B_A")
            pool.starmap(integruj_b_a, partition_count(B_A.objects, num_proc))
            logger.debug("Przypisuje jednostki do autorow")

        if enable_all or options["enable_przypisz_jednostki"]:
            logger.debug("Przypisuje Autor_Jednostka masowo")
            pool.apply(przypisz_jednostki)

        if enable_all or options["enable_dodaj_aktualnosc"]:
            pool.apply(dodaj_aktualnosc)



from django.conf.urls import url

from integrator2.views import Main, UploadListaMinisterialna, DetailBase

urlpatterns = (
        url(r'^upload/$', UploadListaMinisterialna.as_view(), name='upload_lista_ministerialna'),
        # url(r'^nowy_lista_min/$', .as_view(template_name="integrator_upload.html"), name='new_list_min'),
        url(r'^$', Main.as_view(), name='main'),
        url(r'^(?P<model_name>[a-z]+)/(?P<pk>[0-9]+)/$', DetailBase.as_view(), name='detail'),

)



from django.db import transaction

from pbn_api.management.commands.util import PBNBaseCommand

from bpp.models import Dyscyplina_Naukowa, Zrodlo


class Command(PBNBaseCommand):
    """Nadpisuje wszystkie dyscypliny dla zrodel z odpowiednikami z PBNu"""

    @transaction.atomic
    def handle(self, verbosity=1, *args, **options):

        dyscypliny = {x.kod: x for x in Dyscyplina_Naukowa.objects.all()}
        missing = set()
        for zrodlo in Zrodlo.objects.exclude(pbn_uid_id=None):
            zrodlo.dyscyplina_zrodla_set.all().delete()

            areas = zrodlo.pbn_uid.value("object", "areas", return_none=True)
            if not areas:
                continue

            for code in areas:
                kod_dyscypliny = "%i.%i" % (int(code[0]), int(code[1:]))
                if kod_dyscypliny not in dyscypliny:
                    if kod_dyscypliny not in missing:
                        print(f"BRAK KODU DYSCYPLINY: {kod_dyscypliny}")
                        missing.add(kod_dyscypliny)
                    continue

                zrodlo.dyscyplina_zrodla_set.create(
                    dyscyplina=dyscypliny[kod_dyscypliny]
                )



# Generated by Django 3.0.11 on 2021-02-28 12:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("import_dyscyplin", "0018_auto_20200329_1719"),
    ]

    operations = [
        migrations.AlterField(
            model_name="kolumna",
            name="rodzaj_pola",
            field=models.CharField(
                choices=[
                    ("tytuł", "tytuł"),
                    ("nazwisko", "nazwisko"),
                    ("imię", "imię"),
                    ("orcid", "orcid"),
                    ("pbn_id", "pbn_id"),
                    ("nazwa jednostki", "nazwa jednostki"),
                    ("wydział", "wydział"),
                    ("dyscyplina", "dyscyplina"),
                    ("kod dyscypliny", "kod dyscypliny"),
                    ("procent dyscypliny", "procent dyscypliny"),
                    ("subdyscyplina", "subdyscyplina"),
                    ("kod subdyscypliny", "kod subdyscypliny"),
                    ("procent subdyscypliny", "procent subdyscypliny"),
                    ("pomiń", "pomiń"),
                ],
                max_length=50,
            ),
        ),
    ]



# Generated by Django 3.0.14 on 2021-11-03 00:04

from django.db import migrations, models

import bpp.models.cache


class Migration(migrations.Migration):

    dependencies = [
        ("rozbieznosci_dyscyplin", "0012_rozbieznosci_dyscyplin_zrodel"),
    ]

    operations = [
        migrations.CreateModel(
            name="RozbieznosciZrodelView",
            fields=[
                (
                    "id",
                    bpp.models.cache.TupleField(
                        base_field=models.IntegerField(),
                        primary_key=True,
                        serialize=False,
                        size=4,
                    ),
                ),
            ],
            options={
                "verbose_name": "rozbieżność dyscyplin źródeł",
                "verbose_name_plural": "rozbieżności dyscyplin źródeł",
                "managed": False,
            },
        ),
    ]



from django.test import TestCase
from django.urls import reverse


def test_rest_api_jednostka_detail(client, jednostka):
    res = client.get(reverse("api_v1:jednostka-detail", args=(jednostka.pk,)))
    assert res.status_code == 200


def test_rest_api_jednostka_list(client, jednostka):
    res = client.get(reverse("api_v1:jednostka-list"))
    assert res.status_code == 200
